function [dt,G2,IF,IP] = SNG2(IQTT,dpl,framespacing,slicedpl,slicespacing)
% ---
% --- SNG2 calculate the intensity-intensity autocorrelation function
% --- in a multi-tau way using the symmetric normalization
% ---
% --- IQTT needs to be a 4-D tensor:
% --- for kinetics        : [xpixel,ypixel, frames, slices]
% --- for full frame mode : [xpixel,ypixel, frames, 1]
% ---
% ---    - only IQTT is mandatory as a parameter
% ---    - the other parameter are optional and the routine can use 
% ---      standard values, however the 'dt' vector won't be correct
% ---
% --- Michael Sprung
% --- Rev 1.0 $ 2006/11/09 $ based on 'rawg2'
% --- 


% =========================================================================
% --- get frame and slice dimensions from input tensor
% =========================================================================
nframes  = size(IQTT,3)                                                    ; % get the number of full frames
nslices  = size(IQTT,4)                                                    ; % get the number of slices
% =========================================================================
% --- end of get frame and slice dimensions from input tensor
% =========================================================================


% =========================================================================
% --- check input parameter
% =========================================================================
if (nargin <= 4), slicespacing = 0.1                        ; end            % if the slicespacing is not defined set slicespacing == 0.1
if (nargin <= 3), slicedpl     = 4                          ; end            % if the slicedpl is not defined set slicedpl == 4
if (nargin <= 2), framespacing = 2 * nslices * slicespacing ; end            % if the framespacing is undefined set framespacing == 2*nslices*slicespacing
if (nargin <= 1), dpl          = 4                          ; end            % if the delays per level (dpl) is not defined use dpl == 4
% =========================================================================
% --- end of check input parameter
% =========================================================================


% =========================================================================
% --- create delays and time vector & create sliceavg tensor
% =========================================================================
if (nslices > 1)
    slicedelays       = finddelays(nslices,slicedpl,1)                     ;
    nslicedelay       = length(slicedelays)                                ;
    sliceavg(:,:,:,1) = sum(IQTT(:,:,:,:),4) / nslices                     ; % create data tensor with only frame time spacing for later use
    clear j                                                                ;
else
    slicedelays = []                                                       ;
    nslicedelay = 0                                                        ;
end
% ---
framedelays = finddelays(nframes,dpl,1)                                    ;
% ---
if (nslices > 1)
    totaldelays = [slicedelays, framedelays]                               ;
else
    totaldelays = framedelays                                              ;
end
ntotaldelay = length(totaldelays)                                          ;
% ---
dt = zeros(1,length(totaldelays))                                          ; % initialize time vector
if (nslices > 1)
    dt(1:nslicedelay)     = totaldelays(1:nslicedelay)     * slicespacing  ; % for the slicedelays
    dt(nslicedelay+1:end) = totaldelays(nslicedelay+1:end) * framespacing  ; % for the framedelays
else
    dt = totaldelays * framespacing                                        ; % for the framedelays
end
% ---
clear slicedelays slicespacing framedelays framespacing                    ;
% =========================================================================
% --- end of creating delays and time vector
% =========================================================================


% =========================================================================
% --- initialize output tensors G2, IF & IP
% =========================================================================
G2 = zeros(size(IQTT,1),size(IQTT,2),ntotaldelay,'single')                 ; % initialize G2
IF = zeros(size(IQTT,1),size(IQTT,2),ntotaldelay,'single')                 ; % initialize IF (intensities fututre)
IP = zeros(size(IQTT,1),size(IQTT,2),ntotaldelay,'single')                 ; % initialize IP (intensities past)
% =========================================================================
% --- end of initializing output tensors G2, IF & IP
% =========================================================================


% =========================================================================
% --- create message string/switch
% =========================================================================
if (nslicedelay > 1)
    imgunits = 'slices'                                                    ;
else
    imgunits = 'frames'                                                    ;
end
% =========================================================================
% --- end of creating message string/switch
% =========================================================================


% =========================================================================
% --- main loop
% =========================================================================
oldlevel   = 0                                                             ;
for k = 1 : ntotaldelay                                                      % loop over all delays

    % =====================================================================
    % --- check if switch from slices to frames is needed
    % =====================================================================
    if (strcmp(imgunits, 'slices') && (k > nslicedelay) )                    % check: if a switchover from slice averaging to frame averaging is needed
        oldlevel   = 0                                                     ;
        imgunits   = 'frames'                                              ; % switch to frames
        IQTT       = sliceavg                                              ; % overwrite IQTT with sliceavg
        clear sliceavg                                                     ; % clear sliceavg
    end
    % =====================================================================
    % --- end of check if switch from slices to frames is needed
    % =====================================================================

    
    % =====================================================================
    % --- create delay, level & tau
    % =====================================================================
    thisdelay = totaldelays(k)                                             ;
    if strcmp(imgunits, 'slices')
        level  = levelofdelay(thisdelay, slicedpl)                         ; % find the level of the actual delay
    end
    if strcmp(imgunits, 'frames')
        level  = levelofdelay(thisdelay, dpl)                              ; % find the level of the actual delay
    end
    if (level < 2)
        tau = thisdelay                                                    ;
    else
        tau = int32(thisdelay / 2.^(level-1) )                             ; % due to the ongoing data averaging the tau is not equal to the delay
    end
    % =====================================================================
    % --- end of creating delay, level & tau
    % =====================================================================

    
    % =====================================================================
    % --- slice calculation routine
    % =====================================================================
    if strcmp(imgunits, 'slices')
        if ( level > oldlevel && level > 1 )                                 % check: if new delay is on a higher level, than average the data tensor
            if ( mod(size(IQTT,4),2) == 0 )                                  % check if the number of slices is even or odd
                lasteven = size(IQTT,4)                                    ; % calculate last even number
                lastodd  = lasteven -1                                     ; % calculate last usable odd  number
            else
                lasteven = size(IQTT,4) - 1                                ; % calculate last even number
                lastodd  = lasteven -1                                     ; % calculate last usable odd  number
            end
            IQTT(:,:,:,1:lasteven/2)     = ( IQTT(:,:,:,1:2:lastodd )   ...
                                           + IQTT(:,:,:,2:2:lasteven)) / 2 ; % average the data tensor
            IQTT(:,:,:,lasteven/2+1:end) = []                              ; % clear the useless part of the tensor
            clear lasteven lastodd                                         ;
        end
        % ---
        t         = 1 : (size(IQTT,4)-tau)                                 ; % create time vector
        G2(:,:,k) = mean(mean(IQTT(:,:,:,t) .* IQTT(:,:,:,t+tau),4),3)     ; % calculate: <I(q,t) * I(q,t+tau)>_t
        IP(:,:,k) = mean(mean(IQTT(:,:,:,t    ),4),3)                      ; % calculate: <I(q,t)>_t_left 
        IF(:,:,k) = mean(mean(IQTT(:,:,:,t+tau),4),3)                      ; % calculate: <I(q,t+tau)>_t_right
        oldlevel  = level                                                  ; % redefine old level
    end
    % =====================================================================
    % --- end of slice calculation routine
    % =====================================================================

    
    % =====================================================================
    % --- frame calculation routine
    % =====================================================================
    if (strcmp(imgunits, 'frames'))
        if (level > oldlevel && level > 1)                                   % check: if the new delay is on a higher level than the data tensor can be averaged
            if ( mod(size(IQTT,3),2) == 0)                                   % check if the number of frames is even or odd
                lasteven = size(IQTT,3)                                    ;
                lastodd  = lasteven -1                                     ;
            else
                lasteven = size(IQTT,3) - 1                                ;
                lastodd  = lasteven -1                                     ;
            end
            % ---
            IQTT(:,:,1:lasteven/2,1)     = ( IQTT(:,:,1:2:lastodd ,1)   ...
                                           + IQTT(:,:,2:2:lasteven,1)) / 2 ; % average the data tensor
            IQTT(:,:,lasteven/2+1:end,:) = []                              ; % clear the useless part of the tensor
            clear lasteven lastodd                                         ;
        end
        % ---
        t         = 1 : (size(IQTT,3)-tau)                                 ; % create time vector
        G2(:,:,k) = mean(IQTT(:,:,t,1) .* IQTT(:,:,t+tau,1),3)             ; % calculate: <I(q,t) * I(q,t+tau)>_t
        IP(:,:,k) = mean(IQTT(:,:,t    ,1),3)                              ; % calculate: <I(q,t)>_t_left
        IF(:,:,k) = mean(IQTT(:,:,t+tau,1),3)                              ; % calculate: <I(q,t+tau)>_t_right
        oldlevel  = level                                                  ; % redefine old level
    end
    % =====================================================================
    % --- end of frame calculation routine
    % =====================================================================

end
clear IQTT imgunits k level oldlevel t tau thisdelay                       ;
clear slicedpl dpl ntotaldelay totaldelays nslices nframes nslicedelay     ;
% =========================================================================
% --- end of main loop
% =========================================================================


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;
% =========================================================================
% --- end of subfunction finddelays
% =========================================================================


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;
% =========================================================================
% --- end of subfunction indexofdelay
% =========================================================================


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;
% =========================================================================
% --- end of subfunction levelofdelay
% =========================================================================


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;
% =========================================================================
% --- end of subfunction delayofindex
% =========================================================================


% ---
% EOF
