function NX_school
% ---
% --- NX_school : Function to perform some further analysis during the 
% --- summer school experiments. It calls the subfunctions NX_structure
% --- and NX_dynamic. 
% ---
% --- NX_structure Calculate the structure factor of a concentrated
% --- solution of Latex spheres in glycerol assuming the cross section of
% --- a dilute solution contains a constant structure factor
%
% --- NX_dynamic Calculate the structure factor of a concentrated
% --- solution of Latex spheres in glycerol assuming the cross section of
% --- a dilute solution contains a constant structure factor
%
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/08/015 $
% ---

[qi,SFc]      = NX_structure                                               ;
[qc, D0overD] = NX_dynamic                                                 ;



%==========================================================================
% --- figure creation : D(dilute)/D(concentrated)
%==========================================================================
delete(findall(0,'Tag','D0overD'))                                         ;
hFigSchool1 = figure('Name','D(dilute)/D(concentrated)','Tag','D0overD')   ;
figure(hFigSchool1)                                                        ;
% ---
subplot(2,1,1)                                                             ;
plot(qi,SFc,'ro-')                                                         ;
xlim([0 1.05*max(max(qi(:)),max(qc(:)))])                                  ;
ylabel('Structure Factor','Interpreter','latex','FontSize',12)             ;       
set(gca,'xMinorTick','on')                                                 ;
% ---
subplot(2,1,2)                                                             ;
plot(qc,D0overD,'bo-')                                                     ;
xlim([0 1.05*max(max(qi(:)),max(qc(:)))])                                  ;
xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)             ;
ylabel('D0 / D(q)','Interpreter','latex','FontSize',12)                    ;
set(gca,'xMinorTick','on')                                                 ;


%==========================================================================
% --- subfunction NX_structure
%==========================================================================
function [qi,SFc] = NX_structure
% --- NX_structure Calculate the structure factor of a concentrated
% --- solution of Latex spheres in glycerol assuming the cross section of
% --- a dilute solution contains a constant structure factor
%
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/08/015 $
% ---


%==========================================================================
% --- get the names of the ascii files of the cross sections of the 
% --- concentrated & dilute solutions of latex sheres in glycerol
% --- and load them
%==========================================================================
currentPath = pwd                                                          ;
% --- dillute solution
[filenameA, filepathA] = uigetfile(                                     ...
    {'*_IQ.ASCII', 'ASCII Files (*.ASCII)'},                            ...
     'Select I(Q) ASCII Files of the dilute solution',                  ...
     'MultiSelect', 'off')                                                 ;
fileA = fullfile(filepathA,filenameA)                                      ; % the whole filename

% --- start to look for the 2nd file in the 1st files directory
str = ['cd ','''',filepathA,'''']                                          ;
eval(str)                                                                  ;

% --- concentrated solution
[filenameB, filepathB] = uigetfile(                                     ...
    {'*_IQ.ASCII', 'ASCII Files (*.ASCII)'},                            ...
     'Select I(Q) ASCII Files of the concentrated solution',            ...
     'MultiSelect', 'off')                                                 ;
fileB = fullfile(filepathB,filenameB)                                      ; % the whole filename

% --- go back to m-file directory
str = ['cd ','''',currentPath,'''']                                        ;
eval(str)                                                                  ;

% --- load data
A = loadascii(fileA)                                                       ;
B = loadascii(fileB)                                                       ;

clear str filenameA filepathA filenameB fileA fileB                        ;

%==========================================================================
% --- data preparation for the structure factor calculation
%==========================================================================
qd   = A.data(:,1)                                                         ;
Id   = A.data(:,2)                                                         ;

qc   = B.data(:,1)                                                         ;
Ic   = B.data(:,2)                                                         ;

minq = 0.0020                                                              ;
qd   = qd(qd>max(min(qc),minq))                                            ;
Id   = Id(qd>max(min(qc),minq))                                            ;

maxq = 0.011                                                               ;
qi   = qd(qd<min(maxq,max(qc)))                                            ;
Ici  = interp1(qc,Ic,qi)                                                   ; % get interpolated intensities
Idi  = Id(qd<min(maxq,max(qc)))                                            ; % use only identical part of Id

SFc  = Ici ./ Idi                                                          ; % calculate structure factor


%==========================================================================
% --- figure creation
%==========================================================================
delete(findall(0,'Tag','SFvsQ1'))                                          ;
hFigSF_Q1 = figure('Name','structure factor vs. q','Tag','SFvsQ1')         ;
figure(hFigSF_Q1)                                                          ;
loglog(qi,Idi,'ro-',qi,Ici,'bo-')                                          ;
legend('dilute sample','concentrated sample')                              ;
xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)             ;
ylabel('Intensity  [photons/sec]','FontSize',12)                           ;        

delete(findall(0,'Tag','SFvsQ2'))                                          ;
hFigSF_Q2 = figure('Name','structure factor vs. q','Tag','SFvsQ2')         ;
figure(hFigSF_Q2)                                                          ;
plot(qi,SFc,'ro-')                                                         ;
xlim([0 1.05*max(qi)])                                                     ;
spreadSFc = max(SFc(:)) - min(SFc(:))                                      ;
ylim([max(0,min(SFc)-0.1*spreadSFc) max(SFc)+0.1*spreadSFc])               ;
xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)             ;
ylabel('Structure Factor','FontSize',12)                                   ;       


%==========================================================================
% --- ASCII output
%==========================================================================
SF_out = [qi,SFc]                                                          ;
savingname = fullfile(filepathB,'structure_factor.txt')                    ;
save(savingname,'SF_out','-ascii','-double','-tabs')                       ;
clear filepathB


%==========================================================================
% --- subfunction NX_dynamic
%==========================================================================
function [qc, D0overD] = NX_dynamic
% --- NX_dynamic Calculate the structure factor of a concentrated
% --- solution of Latex spheres in glycerol assuming the cross section of
% --- a dilute solution contains a constant structure factor
%
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/08/015 $
% ---


%==========================================================================
% --- get the names of the ascii files of the cross sections of the 
% --- concentrated & dilute solutions of latex sheres in glycerol
% --- and load them
%==========================================================================
currentPath = pwd                                                          ;
% --- dilute solution
[filenameA, filepathA] = uigetfile(                                     ...
    {'*_fitresults.ASCII', 'ASCII Files (*.ASCII)'},                    ...
     'Select ASCII File of the fit result of the dilute solution',      ...
     'MultiSelect', 'off')                                                 ;
fileA = fullfile(filepathA,filenameA)                                      ; % the whole filename

% --- start to look for the 2nd file in the 1st files directory
str = ['cd ','''',filepathA,'''']                                          ;
eval(str)                                                                  ;

% --- concentrated solution
[filenameB, filepathB] = uigetfile(                                     ...
    {'*_fitresults.ASCII', 'ASCII Files (*.ASCII)'},                    ...
     'Select ASCII File of the fit result of the concentrated solution',...
     'MultiSelect', 'off')                                                 ;
fileB = fullfile(filepathB,filenameB)                                      ; % the whole filename

% --- go back to m-file directory
str = ['cd ','''',currentPath,'''']                                        ;
eval(str)                                                                  ;

% --- load data
A = loadascii(fileA)                                                       ;
B = loadascii(fileB)                                                       ;

clear str filenameA filepathA filenameB fileA fileB                        ;


%==========================================================================
% --- data preparation
%==========================================================================
good       = 0.005                                                         ; % good fits have a quality < good !!!  

% --- dilute sample
qd         = A.data(:,2)                                                   ; % q column
Gammad     = A.data(:,8)                                                   ; % Gamma=1/tau column
errGammad  = A.data(:,9)                                                   ; % Error of Gamma
Qualityd   = A.data(:,10)                                                  ; % Fitting quality
% ---
qd         = qd(Qualityd<good)                                             ;
Gammad     = Gammad(Qualityd<good)                                         ;
errGammad  = errGammad(Qualityd<good)                                      ;

% --- concentrated sample
qc         = B.data(:,2)                                                   ; % q column
Gammac     = B.data(:,8)                                                   ; % Gamma=1/tau column
errGammac  = B.data(:,9)                                                   ; % Error of Gamma
Qualityc   = B.data(:,10)                                                  ; % Fitting quality
% ---
qc         = qc(Qualityc<good)                                             ;
Gammac     = Gammac(Qualityc<good)                                         ;
errGammac  = errGammac(Qualityc<good)                                      ;


%==========================================================================
% --- dilute sample : fit of D0
%==========================================================================

% --- data preparation
x = qd.^2                                                                  ;
y = Gammad                                                                 ;
e = errGammad                                                              ;
x = x(:); y = y(:); e = e(:)                                               ;

% =========================================================================
% --- define fitting function, start parameter and [dp niter tol]
% =========================================================================
func    = @line0                                                           ; % handle to gauss function below
free    = 1                                                                ; % all parameter are free
pin     = 1e04                                                             ; % start parameter for levenberg [slope]
fcp     = [1e-01 1e03 1e-09]                                               ; % [dp max. iteration tol]

% =========================================================================
% --- call Levenberg-Marquart algorithm 
% =========================================================================
t1 = clock                                                                 ;
[p,sig,f,covp,corp,r2,rv] = levenbergfit(x,y,e,pin,free,func,fcp)          ;
t  = etime(clock,t1)                                                       ;

DISPLAYon = 0                                                              ;
if ( DISPLAYon == 1 )
    paramnames = {'Diffusion constant [Angstroem^2/s]'}                    ;
    disp(sprintf('\rLevenberg-Marquart algorithm results'))                ;
    disp(sprintf('\rFit duration: %6.4f seconds',t))                       ;
    disp(sprintf('Parameter name  Parameter value  Error of parameter value\r')) ;
    for i = 1 : numel(p)
        disp(sprintf('%33s  %13.3f %13.3f', paramnames{i}, p(i), sig(i)))  ;
    end
end
% --- calculate fitresults for plot
bestfit = line0(x,p)                                                       ;

clear x y e                                                                ;


%==========================================================================
% --- figure creation : GAMMAvsQ
%==========================================================================

% --- data preparation (dilute sample)
x = qd.^2                                                                  ;
y = Gammad                                                                 ;
e = errGammad                                                              ;
x = x(:); y = y(:); e = e(:)                                               ;

% --- data preparation (concentrated sample)
xc = qc.^2                                                                 ;
yc = Gammac                                                                ;
ec = errGammac                                                             ;
xc = xc(:); yc = yc(:); ec = ec(:)                                         ;


delete(findall(0,'Tag','GAMMAvsQ'))                                        ;
hFigGvsQ = figure('Name','Gamma vs. Q^2','Tag','GAMMAvsQ')                 ;
figure(hFigGvsQ)                                                           ;
plot(x,y,'ko',xc,yc,'bo',x,bestfit,'r-','MarkerSize',6,'LineWidth',1.5)    ;

% --- errorbars dilute sample
if ( numel(x) > 0 )
    hold on                                                                ;
    yl = y - e                                                             ;
    yu = y + e                                                             ;
    plot(x(yl>0),yl(yl>0),'kx')                                            ; % plot lower  errorbar values
    plot(x,yu,'kx')                                                        ; % plot higher errorbar values
    for j = 1 : numel(x)
        if ( yl(j) > 0 )
            xj = [x(j),x(j),x(j)]                                          ;
            yj = [yl(j),y(j),yu(j)]                                        ;
            plot(xj,yj,'k-')                                               ; % connect lower & higher errorbar values
        else
            xj = [x(j),x(j)]                                               ;
            yj = [y(j),yu(j)]                                              ;
            plot(xj,yj,'k-')                                               ; % connect higher errorbar values                    
        end
    end
    hold off                                                               ;
clear yl yu xj yj                                                          ;
end

% --- errorbars dilute sample
if ( numel(xc) > 0 )
    hold on                                                                ;
    ycl = yc - ec                                                          ;
    ycu = yc + ec                                                          ;
    plot(xc(ycl>0),ycl(ycl>0),'bx')                                        ; % plot lower  errorbar values
    plot(xc,ycu,'bx')                                                      ; % plot higher errorbar values
    for j = 1 : numel(xc)
        if ( ycl(j) > 0 )
            xcj = [xc(j),xc(j),xc(j)]                                      ;
            ycj = [ycl(j),yc(j),ycu(j)]                                    ;
            plot(xcj,ycj,'b-')                                             ; % connect lower & higher errorbar values
        else
            xcj = [xc(j),xc(j)]                                            ;
            ycj = [yc(j),ycu(j)]                                           ;
            plot(xcj,ycj,'b-')                                             ; % connect higher errorbar values                    
        end
    end
    hold off                                                               ;
clear ycl ycu xcj ycj                                                      ;
end

% --- enter text with fitting parameter to figure
str  = {[sprintf('Diffusion constant = %3.3g',p(1)),' [${\AA}^{2}$/s]']}   ;
text('Position',[0.30 0.90],'String',str,'Interpreter','latex'          ...
    ,'Units','normalized','FontSize',12)                                   ;
% --- label axes
legend('dilute sample','concentrated sample','Location','SouthEast')       ;
xlabel('Q$^{2}$ [{\AA}$^{-2}$]','Interpreter','latex','FontSize',12)       ;
ylabel('${\Gamma} = 1/{\tau}$ [s$^{-1}$]','Interpreter','latex','FontSize',12) ;

clear x y e xc yc ec                                                       ;


%==========================================================================
% --- ASCII output
%==========================================================================

% --- data preparation (D(dilute)/D(concentrated))
qc = qc(:)                                                                 ;
D0 = p(1)                                                                  ; % fitting parameter
D  = Gammac ./ qc.^2                                                       ;
D0overD = D0 ./ D                                                          ;
D  = D(:); D0overD = D0overD(:)                                            ;

D0overD_out = [qc,D0overD,D]                                               ;
savingname = fullfile(filepathB,'D0overD.txt')                             ;
save(savingname,'D0overD_out','-ascii','-double','-tabs')                  ;
clear filepathB


% =========================================================================
% --- define fitting function
% =========================================================================
function y = line0(x,p)
% --- y = gauss(x,p) 
% --- p = [slope]

y= p(1) * x                                                                ;

% ---
% EOF