/*                           -*- Mode: C -*-
 * uread.c 
 *
 * like fread, but will read from a uint8 array of length 8
 * usage: u8read(a, size, precision)
 * a is the the uint8 array
 * size is the number of elements returned, default is end of array
 * precision is one of 'schar', 'uchar, 'int{8,16,32,64}', 
 * 'uint{16,32,64}', 'float{32,64}', 'double'
 * default is 'uchar'
 *
 * Author          : Sridhar Anandakrishnan <sak@essc.psu.edu>
 * Created On      : Mon Jul 29 12:30:39 2002
 * Last Modified By: Sridhar Anandakrishnan
 * Last Modified On: Mon Jul 29 12:55:46 2002
 * Update Count    : 13
 * Status          : Unknown, Use with caution!

    Copyright (c) 2002 Sridhar Anandakrishnan, sak@essc.psu.edu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

/* System Headers */
#include <stdio.h>
#include <math.h>
#include "mex.h"

/* Local Headers */

/* Macros */

/* File-scope variables */

/* External variables */

/* External functions */

/* Structures and Unions */
union U {
  unsigned char UC;		/* 8 */
  unsigned short int USI;	/* 16 */
  unsigned long int ULI;	/* 32 */
  signed char SC;		/* 8 */
  signed short int SSI;		/* 16 */
  signed long int SLI;		/* 32 */
  float F;			/* 32 */
  double D;			/* 64 */
  char in[8];			/* input array */
} u;

enum precs {SCHAR, UCHAR, INT8, INT16, INT32, 
	    UINT8, UINT16, UINT32,
	    FLOAT32, FLOAT64, DOUBLE};
int prsz[]={1, 1, 1, 2, 4,
	    1, 2, 4,
	    4, 8, 8};		/* prsz[prec] */

/* Signal Catching Functions */

/* Functions */

/* Main */

void
mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
  int        i, m, n, len, size, prec, precstrlen;
  unsigned char *pu8, *pd;	/* pointer to input and output */
  char *precstr;		/* precision specifier */
  double *d;			/* return value */

  /* Examine input (right-hand-side) arguments. */
  /*     mexPrintf("\n\nThere are %d right-hand-side argument(s).\n", nrhs); */
  if(nrhs<3)
    prec=UCHAR;
  else {
    precstrlen=(mxGetM(prhs[2]))*mxGetN(prhs[2]) + 1;
    precstr=mxCalloc(precstrlen, sizeof(char));
    if(mxGetString(prhs[2], precstr, precstrlen))
      mexErrMsgTxt("Not enough space\n");

    /* windows c-compiler choked on switch-case?? */
    if(!strcmp(precstr,"schar")) prec=SCHAR;
    else if(!strcmp(precstr,"uchar")) prec=UCHAR;
    else if(!strcmp(precstr,"int8")) prec=INT8;
    else if(!strcmp(precstr,"int16")) prec=INT16;
    else if(!strcmp(precstr,"int32")) prec=INT32;
    else if(!strcmp(precstr,"uint8")) prec=UINT8;
    else if(!strcmp(precstr,"uint16")) prec=UINT16;
    else if(!strcmp(precstr,"uint32")) prec=UINT32;
    else if(!strcmp(precstr,"float32")) prec=FLOAT32;
    else if(!strcmp(precstr,"float64")) prec=FLOAT64;
    else if(!strcmp(precstr,"double")) prec=DOUBLE;
    else  mexErrMsgTxt("Unknown precision specifier");
  }
	 
  if(nrhs<2)
    size=1;
  else
    size=mxGetScalar(prhs[1]);
    
  if(nrhs<1)
    mexErrMsgTxt("\n\nOne input required.");
  m=mxGetM(prhs[0]);
  n=mxGetN(prhs[0]);

  if(m>1 && n>1)
    mexErrMsgTxt("\n\nInput must be uint8 vector");
  len=m>=n?m:n;
  if(len<size*prsz[prec])
    mexErrMsgTxt("\n\nInput vector length must be greater than SIZE*NBYTES_PREC");
    
  /* Examine output (left-hand-side) arguments. */
  /*     mexPrintf("\n\nThere are %d left-hand-side argument(s).\n", nlhs); */

  plhs[0]=mxCreateDoubleMatrix(1, size, mxREAL);
  d = mxGetPr(plhs[0]);
  /* pointer to the first element of the uint8 input array */
  pu8 = mxGetData(prhs[0]);

  /* copy the bytes into the union and then cast them to type double */
  for(i=0;i<size;i++) {
    memcpy(u.in, pu8+i*prsz[prec], prsz[prec]);
    /* windows c-compiler choked on switch-case */
    if(prec==UCHAR || prec==INT8) {		/* SC */
      d[i]=u.SC;
    } else if (prec==INT16) {	                /* SSI */
      d[i]=u.SSI;
    } else if (prec==INT32) {			/* SLI */
      d[i]=u.SLI;
    } else if (prec==UINT8 || prec==UCHAR) {		/* UC */
      d[i]=u.UC;
    } else if (prec==UINT16) {		/* USI */
      d[i]=u.USI;
    } else if (prec==UINT32) {		/* ULI */
      d[i]=u.ULI;
    } else if (prec==FLOAT32)	{		/* F */
      d[i]=u.F;
    } else if (prec==FLOAT64 || prec==DOUBLE) {	/* D */
      d[i]=u.D;
    } else {
      mexErrMsgTxt("Unknown precision");
    }
  }
  /*     mexPrintf("d=%f\n", d[i]); */
}
