function [varargout] = combineg2(varargin)                                     
% ---
% --- COMBINEG2 : This function tries to combine/merge overlapping vectors
% ---
% --- Usage:
% --- [x,y]   = combineg2(x,y)    : default       
% --- [x,y,e] = combineg2(x,y,e)  : sorts also the errors accordingly
% --- [x,y,e] = combineg2(x,y,e,'low2high')
% --- [x,y,e] = combineg2(x,y,e,'high2low')
% ---
% --- The input parameters should be two or three cells.
% --- Each should contain the vectors to merge, e.g. x = {x1;x2;x3;...},
% --- y = {y1;y2;y3;...} [ & e = {e1;e2;e3;...} ].
% --- 
% --- Include an option to decide, which curve is shifted. Options are:
% ---    'low2high' --> low  in x curve is shifted
% --- or 'high2low' --> high in x curve is shifted
% ---
% --- The x vectors need to have overlap on only one side 
% --- for this function to work. This function should work for 
% --- any data sets, which can be shifted by adding or subtracting 
% --- a constant value or using a linear correction to match the data. 
% ---
% --- If the data is meaningfull in a logarithmic x axis, than convert the
% --- x vectors first to logarithmic values before calling this routine and
% --- use the backtransformation afterwards.
% ---
% --- Michael Sprung 20061115
% ---


% =========================================================================
% --- control switches
% =========================================================================
PLOTon   = 0                                                               ;
% ---
minoverlap = 10                                                            ; % minimum points in overlapping region to use linear correction
minslope   = 0.050                                                         ; % minimum slope in overlapping region before using linear correction
smooth     = 1                                                             ; % option to smooth the overlapping parts before linear fit
testplot   = 0                                                             ; % additional test figure


% =========================================================================
% --- input control and preparation
% =========================================================================
if ( numel(varargin) ~= 2 && numel(varargin) ~= 3 && numel(varargin) ~= 4 )
    uiwait(msgbox('Wrong number of inputs','error','modal'))               ;
    return                                                                 ;
end
x = varargin{1}                                                            ;
y = varargin{2}                                                            ;
% ---
if ( numel(varargin) == 2)
    if ( numel(x) ~= numel(y) )
        uiwait(msgbox('Different number of x & y vectors','error','modal'));
        return                                                             ; 
    end
end
% ---
if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
    e = varargin{3}                                                        ;
end
% ---
if ( numel(varargin) >= 3  && ~isempty(varargin{3}) == 1 )
    if (  numel(x) ~= numel(y)                                          ...
       || numel(x) ~= numel(e)                                          ...
       || numel(y) ~= numel(e) )
        uiwait(msgbox('Different number of x, y or e vectors'           ...
                     ,'error','modal'))                                    ;
        return                                                             ; 
    end
end
% ---
n = numel(x)                                                               ; % number of functions to merge
for i = 1 :n
    if (    numel(varargin) == 2                                          ...
       || ( numel(varargin) >= 3 && isempty(varargin{3}) == 1 ) )
        if (  numel(x{i,1}) ~= numel(y{i,1}) )
            uiwait(msgbox('Vector size mismatch','error','modal'))         ;
            return                                                         ;       
        end
    end
    if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
        if (  numel(x{i,1}) ~= numel(y{i,1})                            ...
           || numel(x{i,1}) ~= numel(e{i,1})                            ...
           || numel(y{i,1}) ~= numel(e{i,1}) )
            uiwait(msgbox('Vector size mismatch','error','modal'))         ;
            return                                                         ;       
        end
    end
end
% ---
valid       = 0                                                            ; % define test switch
shiftoption = 0                                                            ; % define default shiftoption to 'low2high'
if ( numel(varargin) == 4 )
    shiftinput = varargin{4}                                               ;
    if ( strcmp(shiftinput,'low2high') == 1 )
        shiftoption = 0                                                    ;
        valid       = 1                                                    ;
    elseif ( strcmp(shiftinput,'high2low') == 1 )
        shiftoption = 1                                                    ;
        valid       = 1                                                    ;
    end
    if ( valid == 0 )
       uiwait(msgbox('Wrong shiftoption! Use default','error','non-modal'));        
    end
end
clear valid shiftinput                                                     ;


% =========================================================================
% --- sort each input vector (each data set should be sorted)
% =========================================================================
for i = 1 : n
    xdummy = x{i,1}                                                        ;
    ydummy = y{i,1}                                                        ;
    if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
        edummy = e{i,1}                                                    ;
    end
    [xdummy,xindex] = sort(xdummy)                                         ;
    ydummy          = ydummy(xindex)                                       ;
    if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
        edummy = edummy(xindex)                                            ;
    end
    x{i,1} = xdummy                                                        ;
    y{i,1} = ydummy                                                        ;
    if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
        e{i,1} = edummy                                                    ;
    end
end
clear xdummy ydummy edummy                                                 ;


% =========================================================================
% --- sort inputs (x vectors with lower values should be at the front) 
% =========================================================================
firstelements = zeros(n,1)                                                 ;
lastelements  = zeros(n,1)                                                 ;
for i = 1 : n
    dummy = x{i,1}                                                         ;
    firstelements(i) = dummy(1)                                            ;
    lastelements(i)  = dummy(end)                                          ;
end

[firstsorted,indexsorted] = sort(firstelements)                            ;
lastsorted = lastelements(indexsorted)                                     ;
clear dummy firstelements lastelements                                     ;

xsorted = cell(n,1)                                                        ;
ysorted = cell(n,1)                                                        ;
if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
    esorted{i,1} = cell(n,1)                                               ;
end

for i = 1 : n
    j = indexsorted(i)                                                     ;
    xsorted{i,1} = x{j,1}                                                  ;
    ysorted{i,1} = y{j,1}                                                  ;
    if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
        esorted{i,1} = e{j,1}                                              ;
    end
end

x = xsorted                                                                ;
y = ysorted                                                                ;
if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
    e = esorted                                                            ;
end

clear xsorted ysorted indexsorted                                          ;
if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
    clear esorted                                                          ;
end


% =========================================================================
% --- create input vectors for control display
% =========================================================================
if ( PLOTon == 1)
    xin = x                                                                ;
    yin = y                                                                ;
end


% =========================================================================
% --- check for overlaps
% =========================================================================
overlap = []                                                               ;
for i = 1 : n-1
    if ( lastsorted(i) > firstsorted(i+1) )
        overlap(i) = 1                                                     ;
    else
        overlap(i) = 0                                                     ;        
    end
end
if ( sum(overlap) == 0 )
  uiwait(msgbox('Data do not overlap! No change!','error','non-modal'))    ;
  return                                                                   ;       
end


% =========================================================================
% --- try to shift curves
% =========================================================================
yCS1 = cell(n,1)                                                           ;
yCS2 = cell(n,1)                                                           ;
dCS1 = cell(n,1)                                                           ;
dCS2 = cell(n,1)                                                           ;
% ---
for i = 1 : n-1
    if ( overlap(i) == 1 )
        % =================================================================
        % --- get the overlapping points of both data sets  
        % =================================================================
        xdummy1 = x{i,1}                                                   ;
        ydummy1 = y{i,1}                                                   ;
        x1      = xdummy1(xdummy1 >= firstsorted(i+1))                     ;
        y1      = ydummy1(xdummy1 >= firstsorted(i+1))                     ;
        % ---
        xdummy2 = x{i+1,1}                                                 ;
        ydummy2 = y{i+1,1}                                                 ;
        x2      = xdummy2(xdummy2 <= lastsorted(i))                        ;
        y2      = ydummy2(xdummy2 <= lastsorted(i))                        ;
        % ---
        clear xdummy1 ydummy1 xdummy2 ydummy2                              ;
        % =================================================================
        % --- find the previous or next for each point   
        % =================================================================
        prevx2tox1 = zeros(numel(x1),1)                                    ;
        nextx2tox1 = zeros(numel(x1),1)                                    ;
        for k = 1 : numel(x1)
            prevx2tox1(k) = nearestpoint(x1(k),x2,'previous')              ;
            nextx2tox1(k) = nearestpoint(x1(k),x2,'next')                  ;
        end
        % ---
        prevx1tox2 = zeros(numel(x2),1)                                    ;
        nextx1tox2 = zeros(numel(x2),1)                                    ;
        for l = 1 : numel(x2)
            prevx1tox2(l) = nearestpoint(x2(l),x1,'previous')              ;
            nextx1tox2(l) = nearestpoint(x2(l),x1,'next')                  ;
        end
        % =================================================================
        % --- calculate a constant offset correction (y1 - int(y2))
        % =================================================================
        counter1 = 0                                                       ;
        diff1    = zeros(numel(x1),1)                                      ;
        for k = 1 : numel(x1)
            if ( isnan(prevx2tox1(k)) == 0 && isnan(nextx2tox1(k)) == 0 )
                diff1(k) = y1(k) - ( y2(prevx2tox1(k))                  ...
                         + ( y2(nextx2tox1(k)) - y2(prevx2tox1(k)) )    ...
                         / ( x2(nextx2tox1(k)) - x2(prevx2tox1(k)) )    ...
                         * ( x1(k) - x2(prevx2tox1(k)) ) )                 ; 
            else
                counter1 = counter1 + 1                                    ;
                diff1(k) = 0                                               ;
            end
        end
        diff1  = diff1(:)                                                  ;
        y1diff = sum(diff1) / (numel(x1) - counter1)                       ;                                               
        % =================================================================
        % --- calculate a constant offset correction (y2 - int(y1))
        % =================================================================
        counter2 = 0                                                       ;
        diff2    = zeros(numel(x2),1)                                      ;
        for l = 1 : numel(x2)
            if ( isnan(prevx1tox2(l)) == 0 && isnan(nextx1tox2(l)) == 0 )
                diff2(l) = y2(l) - ( y1(prevx1tox2(l))                  ...
                         + ( y1(nextx1tox2(l)) - y1(prevx1tox2(l)) )    ...
                         / ( x1(nextx1tox2(l)) - x1(prevx1tox2(l)) )    ...
                         * ( x2(l) - x1(prevx1tox2(l)) ) )                 ; 
            else
                counter2 = counter2 + 1                                    ;
                diff2(l) = 0                                               ;
            end
        end
        diff2  = diff2(:)                                                  ;
        y2diff = sum(diff2) / (numel(x2) - counter2)                       ;
        % =================================================================
        % --- average the interpolated constant offset correction
        % =================================================================
        shift  = 1/2 * (y2diff - y1diff)                                   ; % constant offset correction for correlation functions
        % =================================================================
        % --- calculate a linear offset correction 
        % =================================================================
        xfit  = [x1(1:end-counter1);x2(counter2+1:end)]                    ;
        y1fit = [y1(1:end-counter1)                                     ...
                ;y2(counter2+1:end)-diff2(counter2+1:end)]                 ;
        y2fit = [y1(1:end-counter1)-diff1(1:end-counter1)               ...
                ;y2(counter2+1:end)]                                       ;
        % --- sort vectors    
        [xfit,xind] = sort(xfit)                                           ;
        y1fit       = y1fit(xind)                                          ;
        y2fit       = y2fit(xind)                                          ;
        % --- create column vectors
        xfit        = xfit(:)                                              ;
        y1fit       = y1fit(:)                                             ;
        y2fit       = y2fit(:)                                             ;
        % --- calculate polynominal fit of overlapping data (smoothing)
        if ( smooth == 1 )
            y1fit  = polyval(polyfit(xfit,y1fit,2),xfit)                   ;
            y2fit  = polyval(polyfit(xfit,y2fit,2),xfit)                   ;
        end
        % --- create linear fit (basically g2,2 = a1+a2*g2,1 )
        XFIT1   = [ones(size(y1fit)) y1fit]                                ;
        a       = XFIT1 \ y2fit                                            ;
        XFIT2   = [ones(size(y2fit)) y2fit]                                ;
        b       = XFIT2 \ y1fit                                            ;
        % --- calculate polynominal fit of full data + difference
        yCS1{i,1}   = polyval(polyfit(x{i,1}  ,y{i,1},3)  ,x{i,1})         ;
        yCS2{i+1,1} = polyval(polyfit(x{i+1,1},y{i+1,1},3),x{i+1,1})       ;
        dCS1{i,1}   = y{i,1}   - yCS1{i,1}                                 ;
        dCS2{i+1,1} = y{i+1,1} - yCS2{i+1,1}                               ;
        % ---
        clear xfit y1fit y2fit xind XFIT1 XFIT2                            ;
        % =================================================================
        % --- merge the curves
        % =================================================================
        if ( testplot == 1 && n >= 3 )
            figure(1000), subplot(2,2,2*i-1)                               ;
            if ( shiftoption == 0 )
                plot(x{i,1},yCS1{i,1},'ro',x{i+1,1},yCS2{i+1,1},'bo')      ;
            elseif ( shiftoption == 1 )
                plot(x{i,1},y{i,1},'ro',x{i+1,1},y{i+1,1},'bo')            ;
            end
        end
        % ---
        if ( shiftoption == 0 )
            for j = 1 : i
                if (  numel(x1)-counter1+numel(x2)-counter2>=minoverlap ...
                   && a(2) >= minslope )
                    yCS1{j,1} = a(1) + a(2) * yCS1{j,1}                    ;
%                     y{j,1} = a(1) + a(2) * y{j,1}                          ;
                else
                    yCS1{j,1} = yCS1{j,1} + shift                          ; % only a constant shift of all correlation functions
%                     y{j,1} = y{j,1} + shift                                ; % only a constant shift of all correlation functions
                end
            end
            if ( i == n - 1 )
                for k = 1 : n - 1
                    y{k,1} = yCS1{k,1} + dCS1{k,1}                         ;
                end
            end
        elseif ( shiftoption == 1 )
            if (  numel(x1)-counter1 + numel(x2)-counter2 >= minoverlap ...
                && b(2) >= minslope )
                y{i+1,1} = b(1) + b(2) * yCS2{i+1,1} + dCS2{i+1,1}         ;
%                 y{i+1,1} = b(1) + b(2) * y{i+1,1}                          ;
            else
                y{i+1,1} = y{i+1,1} - shift                                ; % only a constant shift of all correlation functions
            end
        end
        % ---
        if ( testplot == 1  && n >= 3 )
            figure(1000), subplot(2,2,2*i)                                 ;
            if ( shiftoption == 0 )
                plot(x{i,1},yCS1{i,1},'ro',x{i+1,1},yCS2{i+1,1},'bo')      ;
            elseif ( shiftoption == 1 )
                plot(x{i,1},y{i,1},'ro',x{i+1,1},y{i+1,1},'bo')            ;
            end
        end
        % =================================================================
        % --- clear leftover variables
        % =================================================================
        clear counter1 counter2 diff1 diff2 shift                          ;
    end
end
clear yCS1 yCS2 dCS1 dCS2 i j                                              ;

% =========================================================================
% --- create input vectors for control display
% =========================================================================
if ( PLOTon == 1)
    xout = x                                                               ;
    yout = y                                                               ;
end


% =========================================================================
% --- create control plot
% =========================================================================
if ( PLOTon == 1)
    delete(findall(0,'Tag','control figure'))                              ;
    figure('Name','control figure','Tag','control figure')                 ;
    subplot(2,1,1)                                                         ;
    for i = 1 : n
        if ( i == 1)
            semilogx(xin{i,1},yin{i,1},'ko-')                              ;
            title('input functions')                                       ;
        else
            hold on                                                        ;
            plot(xin{i,1},yin{i,1},'ko-')                                  ;
            hold off                                                       ;
        end
    end
    subplot(2,1,2)                                                         ;
    for i = 1 : n
        if ( i == 1)
            semilogx(xout{i,1},yout{i,1},'ko-')                            ;
            title('output functions')                                      ;
        else
            hold on                                                        ;
            plot(xout{i,1},yout{i,1},'ko-')                                ;
            hold off                                                       ;
        end
    end
    clear xin yin xout yout                                                ;
end


% =========================================================================
% --- assign output variables
% =========================================================================
varargout{1} = x                                                           ;
varargout{2} = y                                                           ; 
if ( numel(varargin) >= 3 && ~isempty(varargin{3}) == 1 )
    varargout{3} = e                                                       ; 
end


% =========================================================================
% --- subfunction nearestpoint
% =========================================================================
function [IND, D] = nearestpoint(x,y,m)                                    
%
%   NEARESTPOINT - find the nearest value in another vector
%
%   IND = NEARESTPOINT(X,Y) finds the value in Y which is the closest to 
%   each value in X, so that abs(Xi-Yk) => abs(Xi-Yj) when k is not 
%   equal to j.
%   IND contains the indices of each of these points.
%   Example: 
%      NEARESTPOINT([1 4 12],[0 3]) -> [1 2 2]
%
%   [IND,D] = ... also returns the absolute distances in D,
%   that is D == abs(X - Y(IND))
%
%   NEARESTPOINT(X, Y, M) specifies the operation mode M:
%   'nearest' : default, same as above
%   'previous': find the points in Y that just precedes a point in X
%               NEARESTPOINT([1 4 12],[0 3],'previous') -> [1 1 1]
%   'next'    : find the points in Y that directly follow a point in X
%               NEARESTPOINT([1 4 12],[0 3],'next') -> [2 NaN NaN]
%
%   If there is no previous or next point in Y for a point X(i), IND(i)
%   will be NaN.
%
%   X and Y may be unsorted.
%
%   This function is quite fast, and especially suited for large arrays
%   with time data. For instance, X and Y may be the times of two separate
%   events, like simple and complex spike data of a neurophysiological 
%   study.
%   
%
% Created       : august 2004
% Author        : Jos van der Geest
% Email         : matlab@jasen.nl


error(nargchk(2,3,nargin))                                                 ;

if nargin==2
    m = 'nearest'                                                          ;
else
    if ~ischar(m)
        error('Mode argument should be a string (either ''nearest'', ''previous'', or ''next'')') ;
    end
end

if ( ~isa(x,'double') || ~isa(y,'double') )
    error('X and Y should be double matrices')                             ;
end

% --- sort the input vectors
sz        = size(x)                                                        ;
[x, xi]   = sort(x(:))                                                     ; 
[dum, xi] = sort(xi)                                                       ; % for rearranging the output back to X
nx        = numel(x)                                                       ; 
cx        = zeros(nx,1)                                                    ;
qx        = isnan(x)                                                       ; % for replacing NaNs with NaNs later on

[y,yi]    = sort(y(:))                                                     ; 
ny        = length(y)                                                      ; 
cy        = ones(ny,1)                                                     ;

xy        = [x ; y]                                                        ;
[xy, xyi] = sort(xy)                                                       ;

cxy       = [cx ; cy]                                                      ;
cxy       = cxy(xyi)                                                       ; % cxy(i) = 0 -> xy(i) belongs to X, = 1 -> xy(i) belongs to Y
ii        = cumsum(cxy)                                                    ;  
ii        = ii(cxy==0).'                                                   ; % ii should be a row vector

% --- reduce overhead
clear cxy xy xyi                                                           ;

switch lower(m)
    case {'nearest','near','absolute'}                                       % the indices of the nearest point
        ii = [ii ; ii+1]                                                   ;
        ii(ii==0) = 1                                                      ;
        ii(ii>ny) = ny                                                     ;         
        yy = y(ii)                                                         ;
        dy = abs(repmat(x.',2,1) - yy)                                     ;
        [dum, ai] = min(dy)                                                ;
        IND = ii(sub2ind(size(ii),ai,1:nx))                                ;
    case {'previous','prev','before'}                                        % the indices of the previous points
        ii = [ii(2:end) ii(end)]                                           ;
        ii(ii < 1) = NaN                                                   ;
        IND = ii                                                           ;
    case {'next','after'}                                                    % the indices of the next points
        ii = ii + 1                                                        ;
        ii(ii>ny) = NaN                                                    ;
        IND = ii                                                           ;
    otherwise
        error('Unknown method "%s"',m)                                     ;
end

IND(qx) = NaN                                                              ; % put NaNs back in

if nargout==2                                                                % also return distance if requested
    D    = repmat(NaN,1,nx)                                                ;
    q    = ~isnan(IND)                                                     ;    
    D(q) = abs(x(q) - y(IND(q)))                                           ;
    D    = reshape(D(xi),sz)                                               ;
end
    
IND    = reshape(IND(xi),sz)                                               ; % reshape and sort to match input X
q      = ~isnan(IND)                                                       ; % because Y was sorted, we have to unsort the indices
IND(q) = yi(IND(q))                                                        ;


% ---
% EOF
