function [sig,covp,corp,r2,rv,f] = errorsigma(x,y,err,p,dp,free,func,f)
% =========================================================================
%
% --- ERRORSIGMA computes the sigma for each fitting parameter value
% --- call this function after a fit is performed
% --- [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% --- free is 0 for fixed parameters, 1 for variable parameters
% --- a good choice for dp = 0.01 * p
%
% =========================================================================

% =========================================================================
% --- input preparation
% =========================================================================
x            = x(:)                                                        ; % x data
y            = y(:)                                                        ; % y data
err          = err(:)                                                      ; % error of data
wt           = 0 * y                                                       ; % create weighting vector

selected     = find(err)                                                   ;
wt(selected) = 1 ./ err(selected)                                          ;
wt           = wt(:)                                                       ;

m            = length(x)                                                   ; % number of data points
p            = p(:)                                                        ; % parameters of final fit
free         = free(:)                                                     ; % vector showing which parameters are varying
dp           = dp(:)                                                       ; % a vector telling how much to vary the parameter

if ( nargin <= 8 ) 
    f = []                                                                 ;
end
if ( isempty(f) || (length(f) ~= m) )
    f = feval(func,x,p)                                                    ;
end
f = f(:)                                                                   ;

index = find( ~isnan(x) & ~isnan(y) & ~isnan(wt)& ~isnan(f)             ...
            & ~isinf(f) & ~isinf(y) & ~isinf(wt) & wt)                     ;
        
x  = x(index)                                                              ;
y  = y(index)                                                              ;
wt = wt(index)                                                             ;
f  = f(index)                                                              ;
m  = length(y)                                                             ;
dp = dp .* free                                                            ;

sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
msav = []                                                                  ;

% =========================================================================
% --- data reduction for data sets above 1000 points
% =========================================================================
nreduce = 1000                                                             ;
if ( m > nreduce )                                                           % try to avoid crash if too many points
%     disp('Reducing data for statistics to 1000 points')                    ;
    xsav = x                                                               ;
    ysav = y                                                               ;
    fsav = f                                                               ;
    wsav = wt                                                              ;
    k    = m/nreduce                                                       ;
    k    = floor(k*(1:nreduce))                                            ;
    k    = k(find(k >= 1 & k <= m))                                        ;
    % ---
    x    = x(k)                                                            ;
    y    = y(k)                                                            ;
    f    = f(k)                                                            ;
    if (length(wt))
        wt = wt(k)                                                         ;
    end
    msav = m                                                               ;
    m    = nreduce                                                         ;
end
clear nreduce                                                              ;

% =========================================================================
% --- calculate variance COV matrix & correlation matrix of parameters
% --- & reevaluate the Jacobian at optimal values
% --- The following section is Ray Muzic's estimate for covariance 
% --- and correlation assuming covariance of data is a diagonal matrix 
% --- proportional to diag(1/wt.^2).
% --- cov matrix of data estimate is from 
% --- Bard Equation 7-5-13 and Row 1 of Table 5.1
% =========================================================================
jac = partialderivatives(x,f,p,dp,func)                                    ;
msk = find(dp ~= 0)                                                        ;
n   = length(msk)                                                          ; % reduce n to equal number of estimated parameters
jac = jac(:, msk)                                                          ; % use only fitted parameters

Qinv  = diag(wt.*wt)                                                       ;
Q     = diag((0*wt+1)./(wt.^2))                                            ;
resid = y-f                                                                ; %unweighted residuals

jtgjinv = []                                                               ;
jtgjinv = pinv(jac'*Qinv*jac)                                              ;
if ~isempty(jtgjinv)
    covp = resid' * Qinv * resid / (m-n) * jtgjinv                         ;
    d    = sqrt(abs(diag(covp)))                                           ;
    corp = covp./(d*d')                                                    ;
else
    disp('Warning : Couldn''t compute squared Jacobian inverse matrix!')   ;
    covp = zeros(n,n)                                                      ;
    corp = ones(n,n)                                                       ;
end

stdp = sqrt(diag(covp))                                                    ;

if ~isempty(msav)                                                            % restoring data
    x  = xsav                                                              ;
    y  = ysav                                                              ;
    f  = fsav                                                              ;
    m  = msav                                                              ;
    wt = wsav                                                              ;
end

j   = 1                                                                    ;
sig = zeros(size(p))                                                       ;
for i = 1 : length(stdp)
    while free(j)==0
        j=j+1                                                              ;
    end
    sig(j) = stdp(i)                                                       ;
    j = j + 1                                                              ;
end

r  = corrcoef(y.*wt,f.*wt)                                                 ;
r2 = r.*r'                                                                 ;
rv = sum(((f-y).*wt).^2/length(y))                                         ;


% =========================================================================
% --- subfunction
% =========================================================================
function df_dp = partialderivatives(x,f,p,dp,func)
% =========================================================================
%
% --- y = partialderivatives(x,y,p,dp,func)
% --- Returns the partial derivatives of function 'func'.
% --- 'x'(vect) is x axis values, 'y' is y values,
% --- 'p' and 'dp' are parameters and their variation.
% --- 'func' is the function concerned (y=func(x,p)).
% --- Output 'df_dp' is a vector (or matrix) of partials varying of 'dp'
%
% =========================================================================
[nr,nc] = size(x)                                                          ;
if ( nr < nc )
    x = x'                                                                 ;
end
p0    = p                                                                  ; % save init params
df_dp = zeros(length(x), length(p))                                        ;
i = find((abs(dp) <= 1e-12) | ~isfinite(dp) )                              ; % these parameters should not vary
if ( ~isempty(i) )
    dp(i) = 0                                                              ;
end
for i = 1 : length(p)
    if ( dp(i) ~= 0 )
        p(i) = p(i) + dp(i)                                                ;
        t = feval(func,x,p)                                                ;
        if ( length(t) ~= length(f) )
%             disp('Warn : fix length problem in jacobian')                  ;
            t = t(1:length(x))                                             ;
        end
        df_dp(:,i) = (t(:)-f(:)) / dp(i)                                   ;
        p = p0                                                             ;
    end
end
if (nr < nc)
    df_dp = df_dp'                                                         ;
end


% ---
% EOF
