function systemtick = immsystick(varargin)
% ---
% --- IMMSYSTICK Read systick of each image.
% --- SYSTEMTICK = IMMSYSTICK(FILENAME,STARTINDEX,ENDINDEX)
% ---
% --- Input Argument:
% --- FILENAME    : image file name
% --- STARTINDEX  : start image index number
% --- ENDINDEX    : end image index number
% ---
% --- Output Argument:
% --- SYSTEMTICK     : vector of systemticks (time between frames)
% ---
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2005/12/01 $
% --- $Revision: 1.1 $  $Date: 2006/03/06 $ order change of parameters in imm hearder
% --- $Revision: 1.2 $  $Date: 2006/09/10 $ can handle compressed files
% ---

% =========================================================================
% --- input file preparation
% =========================================================================
file            = varargin{1}                                              ;
immStartIndex   = varargin{2}                                              ;
immEndIndex     = varargin{3}                                              ;
% ---
findUnderscore  = findstr(file,'_')                                        ;
findDash        = findstr(file,'-')                                        ;
findDot         = findstr(file,'.')                                        ;
firstImmIndex   = str2num(file(findUnderscore(end)+1:findDash(end)-1))     ;
lastImmIndex    = str2num(file(findDash(end)+1:findDot(end)-1))            ;


% =========================================================================
% --- if image index is out of range, error and return
% =========================================================================
if (  immStartIndex < firstImmIndex || immStartIndex > lastImmIndex     ...
   || immEndIndex   < firstImmIndex || immEndIndex > lastImmIndex       ...
   || immEndIndex   < immStartIndex )
    error('Image index is out of range.')                                  ;
end


% =========================================================================
% --- correct image indices (basing on the total image numbers in imm file)
% =========================================================================
immStartIndex = immStartIndex - firstImmIndex + 1                          ;
immEndIndex   = immEndIndex   - firstImmIndex + 1                          ;


% =========================================================================
% --- open file
% =========================================================================
[fid,message] = fopen(file)                                                ;        
if ( fid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(fid)                                                            ;
    return                                                                 ;
end


% =========================================================================
% --- check for compression
% =========================================================================
fseek(fid,4,'bof')                                                         ;
compressionflag = fread(fid,1 ,'int')                                      ;
fseek(fid,616,'bof')                                                       ;
immversionflag  = fread(fid,1 ,'int')                                      ;
% ---
compression = 0                                                            ; % assume uncompressed files
if ( compressionflag == 65540 && isempty(findstr(file,'_ucp_')) )            % condition for SMD legacy code
    compression = 1                                                        ;
elseif ( compressionflag == 6 && immversionflag >= 11 )                      % condition for compressed data from Imageserver program
    compression = 1                                                        ;
end


% =========================================================================
% --- for uncompressed data : get the image size & start position
% --- for   compressed data : get start position of image of 'StartIndex'
% =========================================================================
fseek(fid,108,'bof')                                                       ;
rows = fread(fid,1 ,'int')                                                 ;
fseek(fid,112,'bof')                                                       ;
cols = fread(fid,1 ,'int')                                                 ;
fseek(fid,116,'bof')                                                       ;
bytes = fread(fid,1 ,'int')                                                ;

if ( compression == 0 )
    
    if ( immversionflag >= 11 )
        immSize = bytes * rows * cols + 1024                               ; % 'bytes' per pixel
    else
        immSize = 2 * rows * cols + 1024                                   ; % 2 bytes per pixel        
    end
    imageStart = (immStartIndex-1)*immSize                                 ;
    
elseif ( compression == 1 )

    if ( compressionflag == 65540 && isempty(findstr(file,'_ucp_')) )        % condition for SMD legacy code
        imageStart      = 0                                                ;
        for k = 1 : immStartIndex - 1
            fseek(fid,imageStart+152,'bof')                                ;
            dlen = fread(fid,1 ,'int')                                     ; % 'dlen' in this case is the data length in bytes
            imageStart = imageStart + 1024 + dlen                          ;
        end
    elseif ( compressionflag == 6 && immversionflag >= 11 )
        imageStart      = 0                                                ;
        for k = 1 : immStartIndex - 1
            fseek(fid,imageStart+152,'bof')                                ;
            dlen = fread(fid,1 ,'int')                                     ; % 'dlen' in this case contains the # of exposed pixels  
            imageStart = imageStart + 1024 + dlen * ( 4 + bytes )          ;
        end
    end
    
end


% =========================================================================
% --- load elapsed time & systemtick
% =========================================================================
systemtick = zeros(1,immEndIndex-immStartIndex+1)                          ;
% ---
if ( compression == 0 )
    
    for iFrame = 1 : immEndIndex - immStartIndex + 1
        fseek(fid,imageStart+(iFrame-1)*immSize+164,'bof')                 ;
        systemtick(iFrame) = fread(fid,1,'uint32=>double')                 ;
    end
    
elseif ( compression == 1 )
    
    for iFrame = 1 : immEndIndex - immStartIndex + 1
        fseek(fid,imageStart+152,'bof')                                    ;
        dlen = fread(fid,1 ,'int')                                         ;
        fseek(fid,imageStart+164,'bof')                                    ; 
        systemtick(iFrame) = fread(fid,1,'uint32=>double')                 ;
        if ( compressionflag == 65540 && isempty(findstr(file,'_ucp_')) )    % condition for SMD legacy code
            imageStart = imageStart + 1024 + dlen                          ;
        elseif ( compressionflag == 6 && immversionflag >= 11 )              % condition for image server code
            imageStart = imageStart + 1024 + dlen * (4+bytes)              ;
        end
    end
    
end


% =========================================================================
% --- close fid and return
% =========================================================================
fclose(fid)                                                                ;


% =========================================================================
% --- the systemtick can overflow
% =========================================================================
a = 2^32                                                                   ; % overflow value (2^32)
c = find(diff(systemtick) < 0)                                             ; % find positions where an overflow occured
if ( numel(c) > 0 )
    for i = 1 : numel(c)
        systemtick(c(i)+1:end) = systemtick(c(i)+1:end) + a                ; % correct systemtick
    end
end


% =========================================================================
% --- convert systemtick to a time in seconds
% =========================================================================
processorspeed = 3.056e09                                                  ; % enter the processor speed of the data collecting computer (Peridot)
systemtick     = systemtick / processorspeed                               ; % systemtick in [s]


% ---
% EOF
