function out = imoverlaymask(img, mask, color)
%   IMOVERLAYMASK Create a mask-based image overlay.
%
%   OUT = IMOVERLAYMASK(IMG, MASK, COLOR) takes an input image, IMG, and a
%   stack of binary images, MASK, and produces an output image whose pixels
%   in the MASK locations have the specified COLOR(s).
%
%   IMG should be a grayscale or an RGB image of class uint8, uint16, int16,
%   logical, double, or single.
%
%   MASK should be a two-dimensional logical matrix or a stack of multiple
%   2-dimensional logical matrices. E.G.: If two binary mask are provided, 
%   than MASK(:,:,1) should be the first mask and MASK(:,:,2) the 2nd. The
%   masks should be of the same size as the image!!!
%
%   COLOR should be a 1-by-3 vector or a stack of 1-by-3 vectors of values
%   in the range [0, 1]. 
%   E.G.: 
%         color = [1 1 1; 1 0 0; 0 1 0; 0 0 1] would be a stack of the four
%         following colors (white, red, green, blue). In the case the 
%         multiple binary mask are provided these colors are used for the
%         different masks.
%
%   OUT is a uint8 RGB image.
%


% If the user doesn't specify the color, use white.
DEFAULT_COLOR = [1 1 1]                                                    ;
if nargin < 3
    color = DEFAULT_COLOR;
end

% check image input
if ( ndims(img) == 2 || ( ndims(img) == 3 && size(img,3) == 3 ) )
else
   uiwait(msgbox('Wrong image input!','error','modal'))                    ;
   return
end
if ( size(img,1) ~= size(mask,1) || size(img,2) ~= size(mask,2) )
   size(img,1),size(mask,1),size(img,2),size(mask,2)
   uiwait(msgbox('Image and Mask have different dimensions!'            ...
                ,'error','modal'))                                         ;
   return
end    

% Make the uint8 the working data class.  The output is also uint8.
% Initialize the output channels (rgb : red, green, and blue)
if ( ndims(img) == 2 )                                                       % grayscale image
    out_red   = img2uint8(img)                                             ;
    out_green = out_red                                                    ;
    out_blue  = out_red                                                    ;
elseif (ndims(img) == 3 && size(img,3) == 3 )                                % rgb image
    out_red   = rgb2uint8(img(:,:,1))                                      ;
    out_green = rgb2uint8(img(:,:,2))                                      ;
    out_blue  = rgb2uint8(img(:,:,3))                                      ;
end

% get the corresponding uint8 color vector(s)
color_uint8 = color2uint8(color)                                           ;

% Replace output values in the mask locations with the new color value
for i = 1 : size(mask,3)
    if ( i <= size(color,1) )
        if ( ndims(img) == 2 )                                               % grayscale case
            out_red  (mask(:,:,i)==1) = color_uint8(i,1)                   ;
            out_green(mask(:,:,i)==1) = color_uint8(i,2)                   ;
            out_blue (mask(:,:,i)==1) = color_uint8(i,3)                   ;
        elseif (ndims(img) == 3 && size(img,3) == 3 )                        % rgb case
            out_red  (mask(:,:,i)==1) = color_uint8(i,1)                   ;
            out_green(mask(:,:,i)==1) = color_uint8(i,2)                   ;
            out_blue (mask(:,:,i)==1) = color_uint8(i,3)                   ;
        end
    else
        default_color_uint8 = color2uint8(DEFAULT_COLOR)                   ;
        if ( ndims(img) == 2 )                                               % grayscale case
            out_red  (mask(:,:,i)==1) = default_color_uint8(1,1)           ;
            out_green(mask(:,:,i)==1) = default_color_uint8(1,2)           ;
            out_blue (mask(:,:,i)==1) = default_color_uint8(1,3)           ;
        elseif (ndims(img) == 3 && size(img,3) == 3 )                        % rgb case
            out_red  (mask(:,:,i)==1) = default_color_uint8(1,1)           ;
            out_green(mask(:,:,i)==1) = default_color_uint8(1,2)           ;
            out_blue (mask(:,:,i)==1) = default_color_uint8(1,3)           ;
        end
    end
end

% Form an RGB image by concatenating the channel matrices along 3rd dimension.
out = cat(3, out_red, out_green, out_blue)                                 ;


% =========================================================================
% subfunction img2uint8
% =========================================================================
function u = img2uint8(img)
%   IMG2UINT8 Convert image to eight-bit unsigned integers.
%
%   IM2UINT8 takes an image as input, and returns an image of
%   class uint8.  If the input image is of class uint8, the
%   output image is identical to it.  If the input image is of
%   class logical, double or uint16, im2uint8 returns the
%   equivalent image of class uint8, rescaling or offsetting 
%   the data as necessary.
%
%   I2 = IMG2UINT8(I1) converts the intensity image I1 to uint8,
%   rescaling the data if necessary.
%
%   I = IMG2UINT8(BW) converts the binary image BW to a uint8 
%   intensity image, changing one-valued elements to 255. 
%

if isa(img, 'uint8')
    u = img                                                                ; % already in uint8 format
elseif isa(img, 'logical')
    u = uint8(img)                                                         ; % convert to uint8 format
    u(img)=255                                                             ; % set color for all ones to 'white' 
elseif isa(img, 'uint16')
    u = uint8(floor(255/65535*img))                                        ; % convert uint16 to uint8 format
elseif ( isa(img, 'double') || isa(img, 'single') )
    u = uint8(25+floor(199/(max(img(:))-min(img(:)))*(img-min(img(:)))))   ; % shift to positive values and spread over values from [25,225]
end


% =========================================================================
% subfunction rgb2uint8
% =========================================================================
function u = rgb2uint8(img)
%   RGB2UINT8 Convert image to eight-bit unsigned integers.
%
%   RGB2UINT8 takes an image as input, and returns an image of
%   class uint8.  If the input image is of class uint8, the
%   output image is identical to it.  If the input image is of
%   class logical, double or uint16, rgb2uint8 returns the
%   equivalent image of class uint8, rescaling or offsetting 
%   the data as necessary.
%
%   I2 = RGB2UINT8(I1) converts the rgb image I1 to uint8,
%   rescaling the data if necessary.
%

if isa(img, 'uint8')
    u = img                                                                ; % already in uint8 format
elseif isa(img, 'logical')
    u = uint8(img)                                                         ; % convert to uint8 format
    u(img)=255                                                             ; % set color for all ones to 'white' 
elseif isa(img, 'uint16')
    u = uint8(round(255/65535*img))                                        ; % convert uint16 to uint8 format
elseif ( isa(img, 'double') || isa(img, 'single') )
    u = uint8(round(255*img))                                              ;
end


% =========================================================================
% subfunction color2uint8
% =========================================================================
function cu = color2uint8(color)
%   COLOR2UINT8 Convert a color vector or a set of color vectors
%   into eight-bit unsigned integer color vector(s)
%

if isa(color, 'uint8')
    cu = color                                                             ; % already in uint8 format
elseif isa(color, 'logical')
    cu = uint8(color)                                                      ; % convert to uint8 format
    cu(color) = 255                                                        ; % set color for all ones to 'white' 
elseif isa(color, 'uint16')
    cu = uint8(floor(255/65535*color))                                     ; % convert uint16 to uint8 format
elseif ( isa(color, 'double') || isa(color, 'single') )
    cu = uint8(round(255*color))                                           ;
end


% ---
% EOF
