function [p,sig,f,covp,corp,r2,rv]=levenbergfit(x,y,err,pin,free,func,fcp)
% =========================================================================
% ---
% --- LEVENBERG : Marquart Levenberg fit routine (fast, non graphic)
% --- Version 3.beta
% --- Levenberg-Marquardt nonlinear regression of f(x,p) to y(x)
% --- Syntax : [p, sig, f] = levenberg(x,y,err,pin,free,func,{fcp})
% ---          with fcp = [ dp niter stol ]
% ---
% --- Definition of dp, niter & stol:
% ---
% --- dp    : this is the step used to compute the partial derivatives 
% ---         f(x,p+dp) - f(x,p)/dp (in percent of parameter). A reasonable
% ---         value should be below 0.1 (10 %) [When higher, this could
% ---         lead to uncertainties in derivative estimates.
% --- niter : maximum number of iterations 
% --- stol  : sets the convergence tolerancy for the fit. When the fit
% ---         criterion (usually least-square) variation between two fit 
% ---         iterations is less than this value, the fit is considered
% ---         to be stable, and finishes.
% ---
% --- Comment:  A point with no error is not selected!
% --- 
% --- Modified by Michael Sprung (630)-252-0283
% --- Modified by Richard I. Shrager (301)-496-1122
% --- Modified by A.Jutan (519)-679-2111
% --- Modified by Ray Muzic 14-Jul-1992
% --- DFM 24.3.96 % EF 03.98
% ---
% =========================================================================


% =========================================================================
% --- define switches and check inputs
% =========================================================================
flagout   = 0                                                              ;
showgraph = 0                                                              ; % not implemented outside of mfit!
if ( nargin < 7 )
    fcp = [1e-01 1e02 1e-09]                                               ;
end

% =========================================================================
% --- initialize routine
% =========================================================================
if ( length(free) ~= length(pin) )
    free = ones(length(pin),1)                                             ; % set all parameter free
end
if ( length(x) ~= length(y) )
    x = 1:length(y)                                                        ;
end
if ( length(err) ~= length(y) )
    err = ones(length(y),1)                                                ;
end
selected = find(err)                                                       ;

wt = 0 * y                                                                 ;
wt(selected) = 1./err(selected)                                            ;

x        = x(:)                                                            ; % x vector
y        = y(:)                                                            ; % y vector
wt       = wt(:)                                                           ; % weight vector
pin      = pin(:)                                                          ; % parameter vector
free     = free(:)                                                         ; % vector telling which parameter to vary
m        = length(y)                                                       ; % number of data points
n        = length(pin)                                                     ; % number of parameter

dp       = abs(free*fcp(1).*pin)                                           ; % dp vector
i = find(free & (abs(dp) <= 1e-12))                                        ; % for small not fixed params
if ( ~isempty(i) )
  dp(i) = max(2e-12,abs(min(pin(find(pin)))*fcp(1)/1000))                  ;
end
niter    = fcp(2)                                                          ; % maximum number of iterations
stol     = fcp(3)                                                          ; % tolerance value

p    = pin                                                                 ;
sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
f    = []                                                                  ;

if ( isempty(pin) )
    return
end

% =========================================================================
% --- if m <= n : SVD method can't work -> we extend data by interpolation
% =========================================================================
if ( m <= n )
    if flagout
        disp('Warning : need to extend data...')                           ;
    end
    newm = ceil(n/m+1)*m                                                   ;
    newx = linspace(x(1), x(length(x)), newm)                              ;
    newx = newx(:)                                                         ;
    y    = interp1(x,y,newx)                                               ;
    wt   = interp1(x,wt,newx)                                              ;
    wt   = wt(:)                                                           ;
    x    = newx                                                            ;
    m    = newm                                                            ;
    clear newx newm                                                        ;
end

% =========================================================================
% --- set up for iterations (with initial data)
% =========================================================================
f        = feval(func,x,p)                                                 ;
f        = f(:)                                                            ;
wt       = wt / sum(wt)                                                    ;
fbest    = f                                                               ;
pbest    = p                                                               ; % initial function values
r        = wt .* (y-f)                                                     ; % error
sbest    = r' * r                                                          ; % square error
ss       = sbest                                                           ;

pprec    = zeros(n,1)                                                      ;
nrm      = zeros(n,1)                                                      ;
maxstep  = Inf * ones(n,1)                                                 ;
chgprev  = Inf * ones(n,1)                                                 ;

kvg      = 0                                                               ;
epsLlast = 1                                                               ;
epstab   = [.1 1 1e2 1e4 1e6 ]                                             ;

% =========================================================================
% --- do iterations 
% =========================================================================
if ( flagout > 0 )
    disp('* Levenberg-Marquardt nonlinear regression')
%     if ( showgraph )
%         [hmf_ctrl, hmf_data, hmf_pars] = mf_figs                           ;
%         figure(hmf_data)                                                   ;
%         hfit = line(x,f,'erasemode','xor','color','c','Tag','mf_fitline')  ;
%     end
    disp(sprintf('Beginning fit (max %d iterations)',niter))               ;
    disp('--------------------------------------')                         ;
    disp('Iteration  Time(s)  Residual Variance')                          ;
end
tic
for iter = 1 : niter
    pprev = pbest                                                          ; % current params
    prt   = partialderivatives(x,fbest,pprev,dp,func)                      ; % partials of func for x
    r     = wt.*(y-fbest)                                                  ; % error
    sprev = sbest                                                          ; % square error
    sgoal = (1-stol)*sprev                                                 ; % sq error to achieve
    for j = 1 : n                                                            % computes partials norm
        if ( dp(j) == 0 )
            nrm(j) = 0                                                     ;
        else
            prt(:,j) = wt.*prt(:,j)                                        ;
            nrm(j)   = prt(:,j)'*prt(:,j)                                  ;
            if ( nrm(j) > 0 )
                nrm(j) = 1/sqrt(nrm(j))                                    ;
            end
        end
        prt(:,j) = nrm(j) * prt(:,j)                                       ; % normalizes partials
    end
    [prt,s,v] = svd(prt,0)                                                 ; % prt=unit matr, s=eigenval : prt=prt*s*v'.
    s         = diag(s)                                                    ; % get eigenvalues (diagonal of sigma)
    g         = prt'*r                                                     ; % gradient by Gauss-Newton formula.
    for jjj = 1:length(epstab)
        epsL = max(epsLlast*epstab(jjj),1e-7)                              ;
        se   = sqrt((s.*s)+epsL)                                           ;
        gse  = g./se                                                       ;
        chg  = ((v*gse).*nrm)                                              ; % change on params
        ochg = chg                                                         ; % check the change constraints and apply as necessary
        for iii = 1 : n
            if (maxstep(iii) == Inf)
                break                                                      ;
            end
            chg(iii) = max(chg(iii),-abs(maxstep(iii)*pprev(iii)))         ;
            chg(iii) = min(chg(iii), abs(maxstep(iii)*pprev(iii)))         ;
        end
        if ( any(ochg ~= chg) )
            if (flagout > 0)
                disp(['Change in parameter(s): '                        ...
                sprintf(1,'%d ',find(ochg ~= chg)) 'were constrained'])    ;
            end
        end 
        aprec = abs(pprec.*pbest)                                          ;
        if (any(abs(chg) > 0.1 * aprec))                                     % it is only worth to evaluate function if there is some change
            p = chg + pprev                                                ;
            f = feval(func,x,p)                                            ;
            f = f(:)                                                       ;
%             if showgraph
%                 set(hfit,'Ydata',f)                                        ;
%                 drawnow                                                    ;
%             end
            r  = wt .* (y-f)                                               ;
            ss = r' * r                                                    ;
            if ( ss < sbest )
                pbest = p                                                  ;
                fbest = f                                                  ;
                sbest = ss                                                 ;
            end
            if ( ss <= sgoal )
                break                                                      ;
            end
        end
    end
    epsLlast = epsL                                                        ;
    if ( ss < eps )
        break                                                              ;
    end
    aprec = abs(pprec .* pbest)                                            ;
    if ( (all(abs(chg) < aprec) & all(abs(chgprev) < aprec)) )
        kvg = 1                                                            ;
        if (flagout)
            fprintf(1,'Parameter changes less than specified precision\n') ;
        end
        break                                                              ;
    else
        chgprev = chg                                                      ;
    end
    if ( ss > sgoal )
        break                                                              ;
    end
    if ( flagout )
        disp(sprintf('   %3d      %6.2f   %8.3g', iter, toc, ss/length(y)));
    end
end

% =========================================================================
% --- set return values
% =========================================================================
p   = pbest                                                                ;
f   = fbest                                                                ;
ss  = sbest                                                                ;
kvg = ( (sbest > sgoal) | (sbest <= eps) | kvg )                           ;
if ( kvg ~= 1 & flagout )
    disp(' CONVERGENCE NOT ACHIEVED! ')                                    ;
end

if ( nargout > 1 )
    [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,p*0.01,free,func,f)      ;
end


% =========================================================================
% --- subfunction to calculate errorbars of final parameters
% =========================================================================
function [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% =========================================================================
%
% --- [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% --- errorsigma computes the sigma for each parameter value
% --- free is 0 for fixed parameters, 1 for variable parameters
% --- a good choice for dp = 0.01 * p
%
% =========================================================================

% =========================================================================
% --- input preparation
% =========================================================================
m        = length(y)                                                       ;
x        = x(:)                                                            ;
y        = y(:)                                                            ;
wt       = wt(:)                                                           ;
p        = p(:)                                                            ;
free     = free(:)                                                         ;
dp       = dp(:)                                                           ;

if isempty(p)
    return                                                                 ;
end
if ( nargin <= 8 ) 
    f = []                                                                 ;
end
if ( isempty(f) || (length(f) ~= m) )
    f = feval(func,x,p)                                                    ;
end
f = f(:)                                                                   ;
if ( length(wt) ~= length(y) )
    wt = ones(length(y),1)                                                 ;
end

index = find( ~isnan(x) & ~isnan(y) & ~isnan(wt)& ~isnan(f)             ...
            & ~isinf(f) & ~isinf(y) & ~isinf(wt) & wt)                     ;
        
x  = x(index)                                                              ;
y  = y(index)                                                              ;
wt = wt(index)                                                             ;
f  = f(index)                                                              ;
m  = length(y)                                                             ;
dp = dp .* free                                                            ;

sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
msav = []                                                                  ;

% =========================================================================
% --- data reduction for data sets above 1000 points
% =========================================================================
nreduce = 1000                                                             ;
if ( m > nreduce )                                                           % try to avoid crash if too many points
%     disp('Reducing data for statistics to 1000 points')                    ;
    xsav = x                                                               ;
    ysav = y                                                               ;
    fsav = f                                                               ;
    wsav = wt                                                              ;
    k    = m/nreduce                                                       ;
    k    = floor(k*(1:nreduce))                                            ;
    k    = k(find(k >= 1 & k <= m))                                        ;
    % ---
    x    = x(k)                                                            ;
    y    = y(k)                                                            ;
    f    = f(k)                                                            ;
    if (length(wt))
        wt = wt(k)                                                         ;
    end
    msav = m                                                               ;
    m    = nreduce                                                         ;
end
clear nreduce                                                              ;

% =========================================================================
% --- calculate variance COV matrix & correlation matrix of parameters
% --- & reevaluate the Jacobian at optimal values
% --- The following section is Ray Muzic's estimate for covariance 
% --- and correlation assuming covariance of data is a diagonal matrix 
% --- proportional to diag(1/wt.^2).
% --- cov matrix of data estimate is from 
% --- Bard Equation 7-5-13 and Row 1 of Table 5.1
% =========================================================================
jac = partialderivatives(x,f,p,dp,func)                                    ;
msk = find(dp ~= 0)                                                        ;
n   = length(msk)                                                          ; % reduce n to equal number of estimated parameters
jac = jac(:, msk)                                                          ; % use only fitted parameters

Qinv  = diag(wt.*wt)                                                       ;
Q     = diag((0*wt+1)./(wt.^2))                                            ;
resid = y-f                                                                ; %unweighted residuals

jtgjinv = []                                                               ;
jtgjinv = pinv(jac'*Qinv*jac)                                              ;
if ~isempty(jtgjinv)
    covp = resid' * Qinv * resid / (m-n) * jtgjinv                         ;
    d    = sqrt(abs(diag(covp)))                                           ;
    corp = covp./(d*d')                                                    ;
else
    disp('Warning : Couldn''t compute squared Jacobian inverse matrix!')   ;
    covp = zeros(n,n)                                                      ;
    corp = ones(n,n)                                                       ;
end

stdp = sqrt(diag(covp))                                                    ;

if ~isempty(msav)                                                            % restoring data
    x  = xsav                                                              ;
    y  = ysav                                                              ;
    f  = fsav                                                              ;
    m  = msav                                                              ;
    wt = wsav                                                              ;
end

j   = 1                                                                    ;
sig = zeros(size(p))                                                       ;
for i = 1 : length(stdp)
    while free(j)==0
        j=j+1                                                              ;
    end
    sig(j) = stdp(i)                                                       ;
    j = j + 1                                                              ;
end

r  = corrcoef(y.*wt,f.*wt)                                                 ;
r2 = r.*r'                                                                 ;
rv = sum(((f-y).*wt).^2/length(y))                                         ;


% =========================================================================
% --- subfunction
% =========================================================================
function df_dp = partialderivatives(x,f,p,dp,func)
% =========================================================================
%
% --- y = partialderivatives(x,y,p,dp,func)
% --- Returns the partial derivatives of function 'func'.
% --- 'x'(vect) is x axis values, 'y' is y values,
% --- 'p' and 'dp' are parameters and their variation.
% --- 'func' is the function concerned (y=func(x,p)).
% --- Output 'df_dp' is a vector (or matrix) of partials varying of 'dp'
%
% =========================================================================
[nr,nc] = size(x)                                                          ;
if ( nr < nc )
    x = x'                                                                 ;
end
p0    = p                                                                  ; % save init params
df_dp = zeros(length(x), length(p))                                        ;
i = find((abs(dp) <= 1e-12) | ~isfinite(dp) )                              ; % these parameters should not vary
if ( ~isempty(i) )
    dp(i) = 0                                                              ;
end
for i = 1 : length(p)
    if ( dp(i) ~= 0 )
        p(i) = p(i) + dp(i)                                                ;
        t = feval(func,x,p)                                                ;
        if ( length(t) ~= length(f) )
%             disp('Warn : fix length problem in jacobian')                  ;
            t = t(1:length(x))                                             ;
        end
        df_dp(:,i) = (t(:)-f(:)) / dp(i)                                   ;
        p = p0                                                             ;
    end
end
if (nr < nc)
    df_dp = df_dp'                                                         ;
end


% ---
% EOF
