function ascii = loadascii(file)
% --- LOADASCII Load ASCII files created by XPCSGUI.
% ---
% --- The ASCII files from XPCSGUI all have the following structure:
% --- A header, which rows are started with the symbol '#', is followed by
% --- data structured in columns. The last row of the header contains
% --- names for the data columns.
% ---
% --- Input :    file  : file name of ASCII file
% --- Output:    ascii : structure containing the header & data
% ---
% ---            ascii.header     : cell structure with header strings
% ---            ascii.data       : data matrix
% ---            ascii.columnName : cell structure with column names
% --- 
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/02/17 $
% ---


% =========================================================================
% --- clear output structure
% =========================================================================
clear ascii                                                                ; 


% =========================================================================
% --- open ascii file
% =========================================================================
[fid,message] = fopen(file,'r')                                            ;        
if ( fid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(fid)                                                            ;
    return                                                                 ;
end


%==========================================================================
% --- read ascii file of type (header marked by # followed by data)
%==========================================================================
totalLineCounter  = 0                                                      ;
headerLineCounter = 0                                                      ;
dataLineCounter   = 0                                                      ;
% ---
while feof(fid) == 0
    totalLineCounter = totalLineCounter + 1                                ;
    scanline = fgetl(fid)                                                  ;
    if ( scanline(1) == '#' )                                                % belongs to header
        headerLineCounter = headerLineCounter + 1                          ;
        headerfield = ['header{',num2str(headerLineCounter),'}']           ;
        scanline    = scanline(2:end)                                      ; % remove # symbol
        eval(['ascii.',headerfield,'=','''',scanline,'''',';'])            ;
    else                                                                     % belongs to data
        dataLineCounter = dataLineCounter + 1                              ;
        datafield = ['data{',num2str(dataLineCounter),'}']                 ;
        eval(['ascii.',datafield,'=','''',scanline,'''',';'])              ;
    end
end


% =========================================================================
% --- close ascii file
% =========================================================================
fclose(fid)                                                                ; % close file

%==========================================================================
% --- process the ASCII file data
%==========================================================================
for i = 1 :dataLineCounter
    ascii.data{i} = str2num(ascii.data{i})                                 ; % convert string to numbers (row vectors)
end
% ---
noDataColumns = numel(ascii.data{1})                                       ; % find the number of columns in the ascii file
% ---
data = zeros(dataLineCounter,noDataColumns)                                ;
for j = 1 :dataLineCounter
    data(j,:) = ascii.data{j}                                              ; % create data matrix
end
% ---
ascii      = rmfield(ascii,'data')                                         ; % remove the cell structure of the data
ascii.data = data                                                          ; % write data matrix to output structure


%==========================================================================
% --- process the last row of the header containing the data column names
%==========================================================================
columnInfo     = ascii.header{end}                                         ;
% --- try to remove leading and trailing spaces from total string
spacePos     = findstr(columnInfo,' ')                                     ; % find all spaces
while (  numel(spacePos) ~= 0 && spacePos(1) == 1 )
    columnInfo = columnInfo(2:end)                                         ;
    spacePos   = findstr(columnInfo,' ')                                   ; % find all spaces of the new column info string
end
while (  numel(spacePos) ~= 0 && spacePos(end) == numel(columnInfo) )
    columnInfo = columnInfo(1:end-1)                                       ;
    spacePos   = findstr(columnInfo,' ')                                   ; % find all spaces of the new column info string
end
% ---
atLeast2Spaces = findstr(columnInfo,'  ')                                  ; % find positions of at least 2 spaces in the string
breakers       = find(diff(atLeast2Spaces) > 1)                            ; % find positions where the next 2 spaces are separated
for i = 1 : noDataColumns
    if ( i == 1 )                                                            % no leading spaces
        ColumnNames{i} = columnInfo(1:atLeast2Spaces(1) )                  ;
    else
        if ( i == noDataColumns )
            ColumnNames{i} = columnInfo(atLeast2Spaces(end):end )          ;
        else
            ColumnNames{i} = columnInfo(atLeast2Spaces(breakers(i-1))   ...
                                       :atLeast2Spaces(breakers(i-1)+1) )  ;         
        end
    end
end
% ---
for i = 1 : noDataColumns
    columnName = ColumnNames{i}                                            ;
    % --- try to remove leading and trailing spaces from partial strings
    spacePos     = findstr(columnName,' ')                                 ; % find all spaces
    while (  numel(spacePos) ~= 0 && spacePos(1) == 1 )
        columnName = columnName(2:end)                                     ;
        spacePos   = findstr(columnName,' ')                               ; % find all spaces of the new column info string
    end
    while ( numel(spacePos) ~= 0 && spacePos(end) == numel(columnName) )
        columnName = columnName(1:end-1)                                   ;
        spacePos   = findstr(columnName,' ')                               ; % find all spaces of the new column info string
    end
    % ---
    ColumnNames{i} = columnName                                            ;
end
% ---
ascii.columnName = ColumnNames                                             ;


% ---
% EOF
