function [p,sig,f,v,fv,covp,corp,r2,rv]=simplexfit(x,y,err,pin,free,func,fcp)
% =========================================================================
% ---
% --- Simplex routine (see fmins matlab function)
% --- FMINS uses a Nelder-Mead type simplex search method.
% --- Syntax : [p, sig, f]=simplexfit(x,y,err,pin,free,func,{fcp})
% --- with fcp = [ dp niter stol ]
% ---
% --- Definition of dp, niter & stol:
% ---
% --- dp : this indicates the exploration range around starting parameter
% ---      set (in %). For instance, a value of 0.1 will indicate that each
% ---      parameter will first be investigated between +10%/-10% around 
% ---      its starting value. This range can be extended during simplex
% ---      fitting process. Reasonable values should be between 0.1 (10 %) 
% ---      and 0.5(50 %). High values are to be used when your starting 
% ---      parameters are very approximative.
% --- niter : maximum number of iterations 
% --- stol  : sets the convergence tolerancy for the fit. When the fit
% ---         criterion (usually least-square) variation between two fit 
% ---         iterations is less than this value, the fit is considered
% ---         to be stable, and finishes.
% ---
% --- Modified by Michael Sprung (630)-252-0283
% --- EF 03.98
% ---
% =========================================================================
flagout   = 0                                                              ;
showgraph = 0                                                              ;
if ( nargin < 7 )
    fcp = [5e-01 1e03 1e-09]                                               ;
end

% =========================================================================
% --- initialize routine
% =========================================================================
if ( length(free) ~= length(pin) )
    free = ones(length(pin),1)                                             ; % set all parameter free
end
if ( length(x) ~= length(y) )
    x = 1:length(y)                                                        ;
end
if ( length(err) ~= length(y) )
    err = ones(length(y),1)                                                ;
end
selected = find(err)                                                       ;

wt = 0 * y                                                                 ;
wt(selected) = 1./err(selected)                                            ;

x        = x(:)                                                            ;
y        = y(:)                                                            ;
wt       = wt(:)                                                           ;
pin      = pin(:)                                                          ;
free     = free(:)                                                         ;
fixed    = find(~free)                                                     ;
m        = length(y)                                                       ;
n        = length(pin)                                                     ;

dp       = fcp(1) * free                                                   ;
i = find(free & (abs(dp) <= 1e-12))                                        ;  % for small not fixed params
if ( ~isempty(i) )
    dp(i) = max(2e-12,abs(min(pin(find(pin)))*fcp(1)/1000))                ;
end
niter    = fcp(2)                                                          ;
stol     = fcp(3)                                                          ;

p    = pin                                                                 ;
sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
f    = []                                                                  ;
v    = []                                                                  ;
fv   = []                                                                  ;
if ( isempty(pin) )
    return
end

% =========================================================================
% --- if m <= n : SVD method can't work -> we extend data by interpolation
% =========================================================================
if ( m <= n )
    if flagout
        disp('Warning : need to extend data...')                           ;
    end
    newm = ceil(n/m+1)*m                                                   ;
    newx = linspace(x(1), x(length(x)), newm)                              ;
    newx = newx(:)                                                         ;
    y    = interp1(x,y,newx)                                               ;
    wt   = interp1(x,wt,newx)                                              ;
    wt   = wt(:)                                                           ;
    x    = newx                                                            ;
    m    = newm                                                            ;
    clear newm newx                                                        ;
end

% =========================================================================
% --- Set up a simplex near the initial guess.
% --- Some improvements suggested by L.Pfeffer at Stanford
% =========================================================================
xsav   = x                                                                 ; % store the x data in xsav
ysav   = y                                                                 ; % store the y data in xsav

tol    = dp                                                                ;
tol2   = stol * length(ysav)                                               ;

x      = pin                                                               ;
n      = numel(x)                                                          ;
xin    = x(:)                                                              ; % Force xin to be a column vector
v      = xin                                                               ; % Place input guess in the simplex!
x(:)   = v                                                                 ;

f      = feval(func,xsav,x)                                                ;
f      = f(:)                                                              ;
r      = wt .* (ysav-f)                                                    ;
fv     = r'*r                                                              ;
pbest  = x                                                                 ;
ssbest = fv                                                                ;
fbest  = f                                                                 ;
niter  = niter * length(find(dp))                                          ;

if flagout
    disp('*** Nelder-Mead type simplex fit ***')
end

tic
usual_delta     = tol                                                      ; % percent deltas for non-zero terms
zero_term_delta = min(tol(find(tol))) / 50                                 ; % Even smaller delta for zero elements of x
for j = find(dp')
     y = xin;
     if ( y(j) ~= 0 )
         y(j) = (1 + usual_delta(j))*y(j)                                  ;
     else
         y(j) = zero_term_delta                                            ;
     end
     v    = [v y]                                                          ;
     x(:) = y                                                              ;
     f    = feval(func,xsav,x)                                             ;
     r    = wt .* (ysav-f)                                                 ;
     f    = r' * r                                                         ;
     fv   = [fv  f]                                                        ;
     y    = xin                                                            ;
     if ( y(j) ~= 0 )
         y(j) = (1 - usual_delta(j))*y(j)                                  ;
     else
         y(j) = -zero_term_delta                                           ;
     end
     v    = [v y]                                                          ;
     x(:) = y                                                              ;
     f    = feval(func,xsav,x)                                             ;
     r    = wt .* (ysav-f)                                                 ;
     f    = r' * r                                                         ;
     fv   = [fv  f]                                                        ;
     y = xin;
     if ( y(j) ~= 0 )
         y(j) = (1 + 2*(rand-0.5)*usual_delta(j))*y(j)                     ;
     else
         y(j) = -zero_term_delta                                           ;
     end
     v    = [v y]                                                          ;
     x(:) = y                                                              ;
     f    = feval(func,xsav,x)                                             ;
     r    = wt .* (ysav-f)                                                 ;
     f    = r' * r                                                         ;
     fv   = [fv  f]                                                        ;
end

[fv,j]     = sort(fv)                                                      ;
v          = v(:,j)                                                        ;
[np1,n]    = size(v)                                                       ;
n          = n-1                                                           ;
func_evals = n+1                                                           ;

if ( flagout > 0 ) 
    disp(sprintf('*** Beginning fit (max %d iterations)',niter))           ;
    disp('--------------------------------------')
    disp('Iteration  Time(s)  Residual Variance');
    fprintf(1,'   %3d      %6.2f   %8.3f %s\n'                          ...
           ,func_evals,toc,fv(1)/length(ysav),'initial  (best)')           ;
end

alpha = 1                                                                  ;
beta  = 1/2                                                                ;
gamma = 2                                                                  ;
onesn = ones(1,n)                                                          ;
ot    = [2:n+1]                                                            ;
on    = [1:n]                                                              ;

% =========================================================================
% --- Iterate until the diameter of the simplex is less than the tolerance 
% =========================================================================
while ( func_evals < niter )
    % ---
    if ( max(abs(fv(1)-fv(ot))) <= tol2 )
        break
    end
    % ---
    vbar       = (sum(v(:,on)')/n)'                                        ;
    vr         = (1 + alpha)*vbar - alpha*v(:,n+1)                         ;
    vr(fixed)  = pin(fixed)                                                ;
    x(:)       = vr                                                        ;
    f          = feval(func,xsav,x)                                        ;
    f          = f(:)                                                      ;
    r          = wt .* (ysav-f)                                            ;
    fr         = r' * r                                                    ;
    func_evals = func_evals + 1                                            ;
    vk         = vr                                                        ;
    fk         = fr                                                        ;
    how        = 'reflect '                                                ;
    % ---
    if ( fr < fv(n) )
        if ( fr < fv(1) )
            ve         = gamma*vr + (1-gamma)*vbar                         ;
            ve(fixed)  = pin(fixed)                                        ;
            x(:)       = ve                                                ;
            f          = feval(func,xsav,x)                                ;
            f          = f(:)                                              ;
            r          = wt .* (ysav-f)                                    ;
            fe         = r' * r                                            ;
            func_evals = func_evals + 1                                    ;
            if ( fe < fv(1) )
                vk  = ve                                                   ;
                fk  = fe                                                   ;
                how = 'expand  '                                           ;
            end
        end
    else
        vt = v(:,n+1)                                                      ;
        ft = fv(n+1)                                                       ;
        if ( fr < ft )
            vt = vr                                                        ;
            ft = fr                                                        ;
        end
        vc         = beta*vt + (1-beta)*vbar                               ;
        vc(fixed)  = pin(fixed)                                            ;
        x(:)       = vc                                                    ;
        f          = feval(func,xsav,x)                                    ;
        f          = f(:)                                                  ;
        r          = wt .* (ysav-f)                                        ;
        fc         = r' * r                                                ;
        func_evals = func_evals + 1                                        ;
        if ( fc < fv(n) )
            vk  = vc                                                       ;
            fk  = fc                                                       ;
            how = 'contract'                                               ;
        else
            for j = 2 : n
                v(:,j) = (v(:,1) + v(:,j))/2                               ;
                x(:)   = v(:,j)                                            ;
                f      = feval(func,xsav,x)                                ;
                f      = f(:)                                              ;
                r      = wt .* (ysav-f)                                    ;
                fv(j)  = r' * r                                            ;
            end
            func_evals = func_evals + n-1                                  ;
            vk         = (v(:,1) + v(:,n+1))/2                             ;
            vk(fixed)  = pin(fixed)                                        ;
            x(:)       = vk                                                ;
            f          = feval(func,xsav,x)                                ;
            f          = f(:)                                              ;
            r          = wt .* (ysav-f)                                    ;
            fk         = r' * r                                            ;
            func_evals = func_evals + 1                                    ;
            how        = 'shrink  '                                        ;
        end
    end
    v(:,n+1) = vk                                                          ;
    fv(n+1)  = fk                                                          ;
    [fv,j]   = sort(fv)                                                    ;
    v        = v(:,j)                                                      ;

    iter     = func_evals                                                  ;
    ss       = sum(((f-ysav).*wt).^2/length(ysav))                         ;
    if (flagout) 
        fprintf(1,'   %3d      %6.2f   %8.3g %s', iter, toc, ss,how)       ;
    end
    if ( ss <= ssbest ) 
        pbest  = x                                                         ;
        ssbest = ss                                                        ;
        fbest  = f                                                         ;
        if (flagout)
%             if showgraph
%                 mf_upars(x,sig)                                            ;
%                 set(hfit,'Ydata',f)                                        ;
%                 drawnow                                                    ;
%             end
            fprintf(1,' (best)')                                           ;
        end
    end
    if (flagout)
        fprintf(1,'\n')                                                    ;
    end

end

x(:) = v(:,1)                                                              ;
if ( func_evals == niter )
    if flagout
        disp(' Getting best fit... ')                                      ;
    end
    kvg = 0                                                                ;
else
    kvg = 1                                                                ;
end

% =========================================================================
% --- set return values
% =========================================================================
p  = pbest(:)                                                              ;
f  = fbest                                                                 ;
y  = ysav                                                                  ;
x  = xsav                                                                  ;
r  = corrcoef(y.*wt,f.*wt)                                                 ;
r2 = r .* r'                                                               ;
ss = sum(((f-ysav).*wt).^2/length(ysav))                                   ;

% =========================================================================
% --- now evaluates 'v' params dispersion near final solution
% --- v is nparams columns
% =========================================================================
if ( nargout > 1 )
    [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,p*0.01,free,func,f)      ;
else
    sig2 = std(v')                                                         ;
    sig  = sig2(:) .* free(:)                                              ;
end
fv = fv/length(y)                                                          ; % get residual variances
if flagout
    fprintf(1,'Best RV : %.3f - Worse RV : %.3f \n', min(fv),max(fv))      ;
end

% =========================================================================
% --- work out chi squared
% =========================================================================
v     = length(y)-length(find(dp))                                         ; % # of degrees of freedom (#sel points - #free pars)
ChiSq = sum(((y-f).*wt).^2 ) / v                                           ;


% =========================================================================
% --- subfunction to calculate errorbars of final parameters
% =========================================================================
function [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% =========================================================================
%
% --- [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% --- mf_sig computes sigma for each parameter
% --- free is 0 for fixed parameter, 1 for variable
% --- good dp is 0.01
%
% =========================================================================

% =========================================================================
% --- input preparation
% =========================================================================
m        = length(y)                                                       ;
x        = x(:)                                                            ;
y        = y(:)                                                            ;
wt       = wt(:)                                                           ;
p        = p(:)                                                            ;
free     = free(:)                                                         ;
dp       = dp(:)                                                           ;

if isempty(p)
    return                                                                 ;
end
if ( nargin <= 8 ) 
    f = []                                                                 ;
end
if ( isempty(f) | (length(f) ~= m) )
    f = feval(func,x,p)                                                    ;
end
f = f(:)                                                                   ;
if ( length(wt) ~= length(y) )
    wt = ones(length(y),1)                                                 ;
end

index = find( ~isnan(x) & ~isnan(y) & ~isnan(wt)& ~isnan(f)             ...
            & ~isinf(f) & ~isinf(y) & ~isinf(wt) & wt)                     ;
        
x  = x(index)                                                              ;
y  = y(index)                                                              ;
wt = wt(index)                                                             ;
f  = f(index)                                                              ;
m  = length(y)                                                             ;
dp = dp .* free                                                            ;

sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
msav = []                                                                  ;

% =========================================================================
% --- data reduction for data sets of more than 500 points
% =========================================================================
nreduce = 500                                                             ;
if ( m > nreduce )                                                           % try to avoid crash if too many points
%     disp('Reducing data for statistics to 500 points')                     ;
    xsav = x                                                               ;
    ysav = y                                                               ;
    fsav = f                                                               ;
    wsav = wt                                                              ;
    k    = m/nreduce                                                       ;
    k    = floor(k*(1:nreduce))                                            ;
    k    = k(find(k >= 1 & k <= m))                                        ;
    % ---
    x    = x(k)                                                            ;
    y    = y(k)                                                            ;
    f    = f(k)                                                            ;
    if (length(wt))
        wt = wt(k)                                                         ;
    end
    msav = m                                                               ;
    m    = nreduce                                                         ;
end
clear nreduce                                                              ;

% =========================================================================
% --- calculate variance COV matrix & correlation matrix of parameters
% --- & reevaluate the Jacobian at optimal values
% --- The following section is Ray Muzic's estimate for covariance 
% --- and correlation assuming covariance of data is a diagonal matrix 
% --- proportional to diag(1/wt.^2).
% --- cov matrix of data estimate is from 
% --- Bard Equation 7-5-13 and Row 1 of Table 5.1
% =========================================================================
jac = partialderivatives(x,f,p,dp,func)                                    ;
msk = find(dp ~= 0)                                                        ;
n   = length(msk)                                                          ; % reduce n to equal number of estimated parameters
jac = jac(:, msk)                                                          ; % use only fitted parameters

Qinv  = diag(wt.*wt)                                                       ;
Q     = diag((0*wt+1)./(wt.^2))                                            ;
resid = y-f                                                                ; %unweighted residuals

jtgjinv = []                                                               ;
jtgjinv = pinv(jac'*Qinv*jac)                                              ;
if ~isempty(jtgjinv)
    covp = resid' * Qinv * resid / (m-n) * jtgjinv                         ;
    d    = sqrt(abs(diag(covp)))                                           ;
    corp = covp./(d*d')                                                    ;
else
    disp('Warning : Couldn''t compute squared Jacobian inverse matrix!')   ;
    covp = zeros(n,n)                                                      ;
    corp = ones(n,n)                                                       ;
end

stdp = sqrt(diag(covp))                                                    ;

if ~isempty(msav)                                                            % restoring data
    x  = xsav                                                              ;
    y  = ysav                                                              ;
    f  = fsav                                                              ;
    m  = msav                                                              ;
    wt = wsav                                                              ;
end

j   = 1                                                                    ;
sig = zeros(size(p))                                                       ;
for i = 1 : length(stdp)
    while free(j)==0
        j=j+1                                                              ;
    end
    sig(j) = stdp(i)                                                       ;
    j = j + 1                                                              ;
end

r  = corrcoef(y.*wt,f.*wt)                                                 ;
r2 = r.*r'                                                                 ;
rv = sum(((f-y).*wt).^2/length(y))                                         ;


% =========================================================================
% --- subfunction
% =========================================================================
function df_dp = partialderivatives(x,f,p,dp,func)
% =========================================================================
%
% --- y = partialderivatives(x,y,p,dp,func)
% --- Returns the partial derivatives of function 'func'.
% --- 'x'(vect) is x axis values, 'y' is y values,
% --- 'p' and 'dp' are parameters and their variation.
% --- 'func' is the function concerned (y=func(x,p)).
% --- Output 'df_dp' is a vector (or matrix) of partials varying of 'dp'
%
% =========================================================================
[nr,nc] = size(x)                                                          ;
if ( nr < nc )
    x = x'                                                                 ;
end
p0    = p                                                                  ; % save init params
df_dp = zeros(length(x), length(p))                                        ;
i = find((abs(dp) <= 1e-12) | ~isfinite(dp) )                              ; % these parameters should not vary
if ( ~isempty(i) )
    dp(i) = 0                                                              ;
end
for i = 1 : length(p)
    if ( dp(i) ~= 0 )
        p(i) = p(i) + dp(i)                                                ;
        t = feval(func,x,p)                                                ;
        if ( length(t) ~= length(f) )
%             disp('Warn : fix length problem in jacobian')                  ;
            t = t(1:length(x))                                             ;
        end
        df_dp(:,i) = (t(:)-f(:)) / dp(i)                                   ;
        p = p0                                                             ;
    end
end
if (nr < nc)
    df_dp = df_dp'                                                         ;
end


% ---
% EOF
